// ---------------------------------------------------------------
// subsystem           : SMAP L4_C
// module              : l4c_DAILY.h
// purpose             : header containing macros, abstract-data-types
// version             : v7.4.3 -- mod to substitute use of frm_utilAssignBits
// for frm_utilSetBits in pre,post proc version             : v7.4.2 -- mod to
// add corrected write of missing_value version             : v7.4.1 -- mods for
// HDF5 v1.10.6 hid_t compatibility version             : v7.3.0 -- see
// L4C_VERS_TAG in l4c_DAILY.h for latest invocation          : ./l4c_DAILY.ex
// l4c_DAILY.rip research code author: Lucas Jones production code and
// integration: Joe Glassy, Lupine Logic Inc. dependencies        : In the RIP
// session configuration file, the
//                       L4C_INPUT_DATASET_xxx HDF5 dataset names are NOW
//                       MANDATORY as these are fully utilized by the pre
//                       processor logic
// dependencies        : macros: HAVE_UUID (=1) if available, L4_C=1 if in L4_C
// environment,
//                       HDF4_REQ (only if needed), HDF5_REQ
// revised             : v7.4.3-2021-10-25T15:01, replace SetBits with
// frm_utilAssignBits revised             : v7.4.1-2020-07-14, added
// INT64_FILL_VALUE, H5T_NATIVE_LLONG, due to hid_t and int64_t prevalence now
// revised             : v7.3.0-2017-06-19T10:40:00, tag 7.3.0, identified and
// corrected horizontal striping
//                       error at 30deg N lat, caused by incorrect use of 1D
//                       sparse smrz ancillary images where the 2D smrz SHOULD
//                       have been used in the SOC ancillary file.
// revised             : v7.3.0-2017-06-07T18:26:00, tag as 7.3.0, added
// sesCtl_p->bplut_uuid (string) and
//                       sesCtl_p->bplut_md5 (string), parameterized by optional
//                       RIP NVPs L4C_BPLUT_UUID and L4C_BPLUT_MD5 respectively.
//                       If present, these are stored and echoed to log file. No
//                       explicit runtime check on these is implemented so these
//                       currently represent only a "soft" consistency check,
//                       where a strictly enforced (exe halt) MAY be implemented
//                       in the future if the BPLUT's internal UUID doesn't
//                       match the UUID in NVP L4C_BPLUT_UUID etc. Changed call
//                       interface to BPLUT_read fx.
// revised             : v7.2.1-2017-05-24T17:05, in l4c_DAILY_PostProcessor.c
// Lucas also
//                       corrected RMSE QA var initialization to missing -9999.0
//                       values for the RMSE average as well as RMSE PFT wise QA
//                       vars.
// revised             : v7.2.1-2016-05-24T14:44, in module file
// l4c_DAILY_PostProcessor.c,
//                       where EC/emult_mean and other EC vars were scaled by
//                       100.0, we previously didn't skip over -9999.0 valued
//                       pixels as we should have. This has now been changed (ca
//                       line 944) such that -9999 valued pixels are now
//                       correctly left intact and NOT scaled. Symptom:
//                       Incorrectly scaling them resulted in bad -999900.0
//                       valued pixels.
// revised             : v7.1.4-2016-12-07T16:43:00, minor doc addition only, no
// code change.
//                       Do note use of gcc -->
//                       /opt/rh/devtoolset-2/root/usr/bin/gcc -std=c99 for the
//                       default build at NTSG on stout.ntsg.umt.edu
// revised             : v7.1.3-2016-04-13T11:23:00 <JMG> in
// l4c_DAILY_WriteMetadata.c module
//                       a reported missing comma after "2s" inserted into the
//                       "flag_masks" string, responding to a bug report by
//                       Scott Dunbar, Ed Armstrong and Christin Alarcon (issued
//                       Feb 9th 2016 that I only now received word of). Note
//                       since these are NOT used or assigned in L4_C we should
//                       look into removing them entirely unless (as
//                       placeholders only) they help compatibility.
// revised             : v7.1.2-2016-03-16T16:04:00, CLOSED-- investigated and
// resolved rh_mean ZERO on output bug.
//                       STATUS: bug corrected; it was caused by RIP file
//                       mis-configuration, where SOC dataset names in RIP files
//                       for Tv2001 did not match those in actual SOC restart
//                       file. Once they match, problem is resolved.  No code
//                       changes required at this point.
// revised             : v7.1.1-2016-03-04T12:16:00, corrected a RMSE bug
//                        (allowing neg values) identified and corrected
//                         in l4c_DAILY_ModMule.c by Lucas, and added QA
//                         Bitfield legend back in , as this was mistakenly
//                         deprecated when carbon_model_qual was deprecated. and
//                         added optional dataset attrib writes of observed
//                         min,max,mean
// history            : v7.1.1-2016-03-02T09:59, investigate potential "negative
// RMSE" issue
//                       in either PostProcessor, or mod_mult/mod_func. In
//                       progress
// history            : v7.1.0-2016-02-18T13:55, perform final tests and prep
// of 7.1 RSME enabled delivery
//                       and harmonized more module and header file paths to use
//                       common prefix l4c_DAILY_{xxx} and changed all EC
//                       mult_av names to EC mult_mean to match Lucas names
// history            : v7.0.9-2016-02-17T10:02, corrected qa_count _Fillvalue
// datatype assignment history            : v7.0.8-2016-02-11T14:48:00, changed
// spelling of
//                       _Fillvalue to _FillValue and re-factored _FillValue
//                       writes to use new l4c_Hdf5_AssignFillValue() type-aware
//                       function.
// history            : v7.0.8-2016-02-10T16:10:00, deprecated output of QA
// carbon_model_qual_pft{1..8}
//                       as these are NOW REPLACED by RMSE QA output variables
// history            : v7.0.7-2016-01-11, <JMG> in
// l4c_DAILY_Hdf5_Read_and_Updates.c in
//          int l4c_SOC_or_FT_CalendarUpdate(), added logic to AUTO-RESET the FT
//          and SOC annual Day-of-Year {1..366} calendar slots to 0 when January
//          1 encountered since otherwise they essentially overwrite(lose track)
//          of the current annual periods state. This logic still isn't perfect
//          for multi-year running, but..its an improvement.
//
//          RIP change: # all prior L4C_OUTPUT_DATASET_xx nvps have been fully
//          deprecated as of 1/1/2016.
//
// history           : v7.0.6-2016-01-07T14:22, added l4c_SesCtl_Display() for
// diagnostics and
//                       continued RMSE development. Bug fix: increased array
//                       size of Hdf5GridPropAdt's dset_L4C_IdAry[] from 64 to
//                       128 -->  dset_L4C_IdAry[128]
// history             : v7.0.5-2016-01-04T14:11, RMSE development work
// continued history             : v7.0.4-2015-12-21T16:41:00, re-verified that
// FPAR CLIM pixels
//                       are substituted into "fpar" regardless how the
//                       L4C_GPP_METHOD RIP switch is set, which correctly
//                       matches the science code logic. Next, resumed work on
//                       implementation of RSME QA using (nee_qa). Adding RMSE
//                       will change the L4_C granule structure, thus require an
//                       updated version ID. This mainly involves code in
//                       l4c_PostProc9km_func.c and l4c_DAILY_PkgMgrInit.c,
//                       though other module files will also require
//                       modification (memory mgmt, release, logging, etc)
//
// history            : v7.0.3-2015-11-11T15:44 in l4c_DAILY_WriteMetadata,
// suppress writing
//                       out the CRID from the RIP NVP which is never correct or
//                       kept up to date by OPS anyway.
// history            : v7.0.2-2015-10-15T12:44:00, investigating non-fatal
// valgrind warnings/errors
//                       so, added pInv.invalidSocMean counter,
//                       (l4c_PostProc9km_func.c ca line 1110) may be
//                       false-positives; issue left open for now.
// history            : v7.0.1-2015-09-16T16:22:00, in l4c_DAILY_HDF5.c
// standardized to use
//                       reference to L4C_OUTPUT_VARIABLE_xxx for lat,lon
// history            : v7.0-2015-08-26T16:40:00, Noteworthy change: the SMAP
// radar instrument
//                        failed in early to mid July, so this release
//                        implements a work-around where we use GEOS5 FP "TS"
//                        and a 0 degree threshold as a substitute freeze-thaw
//                        classifier. <=0, its frozen, >0 its thawed. This
//                        version also incorporates new soil moisture rootzone
//                        (smrz) min,max saturation correction logic by Lucas.
//                        A next version will implement the new RMSE logic,
//                        whereupon we add the float32 RMSE, but deprecate the
//                        (8) PFT wise QA uint8 variables, and likely ADD (8)
//                        float32 PFT wise RMSE variables.
// history            : v7.0-2015-09-08T14:22:00-07:00 v 7 DELIVERY MADE TO NCCS
// Discover. history            : v7.0-2015-09-07T21:36:00, established
// equivalence of DoUseFT_GEOS5
//                       and DoUseFT_ActiveTsurf in l4c_DAILY_ModFunc.c since
//                       both write F/T state to bit0 of FT_STATE_UM_M03.  And,
//                       corrected freq reference in l4c_SummarizeFreezeThaw(),
//                       array indices were backwards,fixed.
// history            : v7.0-2015-09-06T14:31:00-07:00, incorporated smrz
// min,max sat
//                       corrections from v6 branch of L4_C sources, and prep
//                       for v7 delivery continues, and changed session LOG file
//                       name to include a YEAR DAY (or day of year) index "DDD"
//                       , e.g. "SMAP_L4_C_MDL_YD210_2015-09-06T17-33-02.log"
//                       and added char  ftMethodString[16] to store FT Method
//                       string for added clarity
// history            : v7.0-2015-07-29T10:22:00, New L4_C fork, after SMAP
// radar failed,
//                       allows use of GEOS5-FP TS driven Freeze/Thaw at 3KM in
//                       place of missing L3_SM_A 3KM F/T
// history            : v6.0-2015-06-01T23:55:00, prep for V10002 delivery.
// history            : v6.0-2015-05-28T16:20:00, add macro L4C_BPLUT_N_COLUMNS
// 44 to l4c_DAILY.h history            : v6.0-2015-05-27T16:35, changed smrz
// min,max names to retrieve
//                       from RIP file using dynamic lookups, via callback calls
//                       like dsetName_p =
//                       nsbGetValueByKey(nsbObj_p,"L4C_INPUT_DATASET_SOILROOTZONE_M09_MIN",
//                       nvpBuf);
// history            : 2015-05-27T15:41, passive change. In
// l4c_PreProc1km_func.c, in
//                       l4c_PrepDeflate_AssignDatasetProperties() for smrz
//                       min,max corrected Nelem from LDOM1KM to correct LDOM9KM
//                       in
//                        pkgMgr_SetObjNelem(pObj_p, EGv2_LDOM9KM )
// ----------OPERATIONS ERA (any historys above) ----------------------
// history            ; v5.x-2014-11-20T14:06:00, changes: refactoring overally
// broad SesCtl into
//                       narrowly focused, major parts, esp segregate all HDF5
//                       file_id's,and PkgMgr and NSB master ptrs, and all
//                       paths. Task in progress, and is NOT yet implemented, so
//                       that prior proven logic is still in place.
// history             : v5.x-2014-11-14TT13:14, changes: corrected
// fpar_qa|fpar_qc error in preprocessor
//                       and implemented provisonal QA RMSE logic in
//                       postprocessor, disabled by default, only enabled via
//                       (optional)  RIP keyword L4C_RMSE_ENABLE = YES switch
//                       for now
// history             : v5.x-2014-11-11TT16:44:00, changes: corrected
// INPUTPOINTER_FPAR when CLIM option
//                       is used and no FPAR 8DAY is specified at Ops, and,
//                       added ft_calendar_cumulative[],soc_cumulative[] doy
//                       slot arrays to better track if a given DOY has been
//                       redundantly been run, noting this cannot be fully
//                       implemented until the newest SOC file with these
//                       attributes is enabled. Other changes: In the
//                       tcf_struct.h header, renamed (4-state) metObj.gppMethod
//                       to metObj.gppInputUsed to better distiguish it from RIP
//                       files (3-state) L4C_GPP_SOURCE, and, corrected
//                       {curly-brace} error in units string of
//                       CompletenessOmission, and added ADT PrePostImagePkgAdt,
//                       to facilitate (future) need to implement the
//                       multiple-day mode. In l4c_DAILY.c added placeholder
//                       logic (not yet enabled) for future multi-day runs,
//                       added enclosing day loop. Note this primitive multi-day
//                       logic will need refinement on the memory-mgmt, to take
//                       advantage of arrays we can allocate once only and
//                       re-use from day-to-day
//
// history              : 2014-11-06T13:24:00, added ADT PrePostImagePkgAdt, to
//                       facilitate (future) need to implement the multiple-day
//                       mode. In l4c_DAILY.c added placeholder logic (not yet
//                       enabled) for future multi-day runs, added enclosing day
//                       loop. Note this primitive multi-day logic will need
//                       refinement on the memory-mgmt, to take advantage of
//                       arrays we can allocate once only and re-use from
//                       day-to-day
//
//                      : and, 2014-11-06, <JMG>  when L4C_GPP_SOURCE = CLIM
//                      (forced
//                        use of only FparClimatology) added SKIP-LOGIC to avoid
//                        pre-processing fpar_M01,fpar_qc_M01 and/or NDVI
//                        since in this case these are not used or relevant.

// history             : v5.0.1-R11E-2014-10-31T16:05, in l4c_FparQaBinUtil.c,
// made
//                       explicit assignments of (scalar) fpar_qa where before
//                       some cases relied on implict logic.  Also in this
//                       batch,streamlined ISO DomainConsistency, and
//                       CompletenessOmission metrics via generation with Lucas,
//                       modified varFlt32 ADT in tcf_struct.h, and LAJ
//                       corrected minor QA bit assignment {0|1} error
// history             : 2014-10-28T16:28:00,fwd step opt,
// history             : 2014-10-27T corrected QaScore scaling
//                       edits of l4c_DAILY_tcfModQa.c and any other involved
//                       modules.
// history             : 2014-10-22T14:16, added attribute L4C_FT_SOURCE, using
// updated fx,
//                       l4c_writeNvpAsAttribute(file_id,"L4C_FT_SOURCE","L4C_FT_SOURCE");
//
// history             : 2014-10-21T21:52:00, added L4_C_GPP_SOURCE as file
// metadata
//                       and enabled RIP "CLIM" or "FPARCLIM" setting for
//                       GPP_SOURCE to allow forcing use of FparClimatology for
//                       devel tests.
// history             : 2014-10-21T16:10, in l4c_MET.ex corrected
// FT_STATE_UM_M03 handling
//                       where the ft_tsurf and ft_radar bits read were off by 1
//                       bit
//                       --they should now be bits 7 for ft_radar and bit 8 for
//                       ft_tsurf. which in turn resulted in faulty EC
//                       frozen_area patterns.
//                     : and, in post processor, changed to rescale (3) EC
//                     1D-sparse vector
//                        Emult,Tmult,Wmult 1D vector arrays from {0.0<= a
//                        <=1.0} to {0.0< a < 100.0} prior to post processing
// history             : 2014-10-16T18:33:00, added PostProcFreqVerifyMetrics
// structure
//                       for local devel phase diag to track QaScore and PftDom
//                       bitflag assignments and may be extended later for other
//                       diagnostic verifications.
// history             : 2014-10-08T16:26:00-07:00-R11E, added INPUTPOINTER_ANC,
//                       INPUTPOINTER_BPLUT, INPUTPOINTER_FPARCLIM to match PSD
// history             : 2014-10-06T16:48:00, added structure
// L4C_ISO_MetaElements to add
//                       missing ISO element Joe A reported, continue ORT prep.
// history             : 2014-10-01T15:07:00-07:00, continued minor clean ups
// for ORT,L4_C Delivery review history             : 2014-09-30, refactored SOC
// update code, split into sep fx. history             : 2014-09-28T22:52:00,
// correcting smrz_mn,mx error in PreProc_Func.c
//                       and added in RIP file L4C_ENABLE_SOC_UPDATE YES|NO
// history             : 2014-09-12T17:34:00, uses UM FRM v1.8.2, and added
//                       sesCtl_p->productionLevel {local|production}
// history             : 2014-09-10T16:44:00, continue implementing ISO
// file-level metadata additions
//                       as per Joe A. requests.
// history             : 2014-09-05T10:52:00, refining internal documentation
// only history             : 2014-09-05T10:06:00, MAJOR CHANGE, consolidated
// all
//                       QA data into a new QA group in primary "mdl" file,
//                       and deprecated ".qa" file entirely.
// history             : 2014-08-25T23:04:00, in l4c_DAILY_rip.c added new
//                       int l4c_SetFparClimatologyDatasetName() to correct
//                       fetch of FparClimatology dataset name based built-in
//                       LUT and period index.
// history             : 2014-08-22T02:07:00, added NDVI_gain,NDVI_offset to run
// params history             : 2014-08-21T22:08:00, wrote work-around to
// SEGFAULT, still
//                       investigating cause.
// history             : 2014-08-21T19:44, found SEGFAULT in
// l4c_PostProc_fltVar9km() in l4c_SetBit()
//                       for nee_mean on just the NDVI test, so added tracer
//                       PostInvalid obj to help find it
// history             : 2014-08-21T15:54:00, NDVI pre-process refined, prep for
// end-to-end
//                       test of l4c_MODIS|NDVI path into l4c_DAILY for pre-ORT.
// history             : 2014-08-15T11:15:00, added l4c_writeNvpAsAttribute() as
//                       borrowed from l4c_MODIS code set.
// history             : v5.0.1-2014-08-14T15:48:02:00-07:00, corrected spelling
// of L4C_PRODUCTION_DATETIME
//                       and added l4c_Ses_DatestampLog(), and
//                       pkgMgr_GetNameByIndex() and now uses FRM API v1.7.8
//                       which supports either DNA or user's name in
//                       memObjFree() and, includes support for
//                       pkgMgr_GetMgr_AllNames() and
//                       pkgMgr_GetThemes_ChildListOfNames()
// history             : v5.0.1-2014-08-08T17:48:00-07:00, <JMG> added
// soc_group_id ... history               v5.0.1-2014-08-07T17:30 <JMG> and
// <LAJ>, plugged FT multiplier from ftRolling into
//                       model dynamics and QA. Added associated Adt and BPLUT
//                       fields.
// history             : v5.0.1-2014-07-25T17:44:00, merged ALL recent LAJ and
// JMG changes
//                       and changed gppFpar_or_NDVI to fparState, after
//                       re-compiling for new interim L4_C v5.0.1 milestone.
// history             : v5.0.1-2014-07-25T16:13, split out RIP fx into new
// module file
//                       l4c_DAILY_rip.c, corrected NDVI hard-wired switch in
//                       ModFunc.c, and changed uint16 "surface_flag" processing
//                       to uint8 "ft_state_l4c_M03" to carry JUST these 3km
//                       bits into L4_C: RADAR FT{0|1}, TSURF_FT{0|1},
//                       RADAR_FT_QC{0|1}, and FT_IN_SWATH{0|1}. Note new RIP
//                       nvp L4C_FT_SOURCE choices: {USE_TSURF_FT|USE_RADAR_FT}
//
// history             : v5.0.1-2014-07-23T19:14:00 <LAJ> added QA bitfield
// logic and
//                       <JMG> added l4c_ParseRipDatasetProperties()
//                       l4c_SesCollectOutputNVPs() and l4c_GetRunParam() for
//                       assisting valid_min,valid_max NSB fetches and added
//                       l4c_AssignBits() to enable
//                       QA bitfield assignments, and <LAJ> added new fwd step
//                       loop aggregations and new post-processor logic to
//                       populate QA bitflag var.
// history             : v5.0.1-2014-06-24T17:29:00, added
// l4c_DAILY_WriteMetadata.c, and
//                       added creation of formal metadata /Metadata,
//                       "group-of-groups" and child groups and attribute
//                       hierarchies.
// history             : v5.0.1-2014-06-17T16:31:00, added
// sesCtl.hdf5ErrStackState, and
//                       updated to use new form of frm_hdf5_ReadAttribute()
//                       call, and updated BPLUT, now at
//                       ./ANC/SMAP_BPLUT_2014-02-18.csv and uses FRM_API v1.7.1
//                       with frm_hdf5_CreateGroup() and new attribute, updated
//                       l4c_DAILY_hdf5.c to prep for writing CF attributes..
//                       writer, int frm_hdf5_WriteAttributeAsType().
// history             : 2014-05-21T15:50:00, changed L4C_DAILY_RELEASE_TAG to
// L4C_SUBSYS_RELEASE_TAG
//                       to better communicate the correspondence between the
//                       set of (3) executables (l4c_MET,l4c_MODIS,l4c_DAILY)
//                       and the tightly coupled files, with a specific
//                       configuration, these depend on.
// history             : 2014-05-19T17:59:00, in l4c_GridPre_Deflate.c and
// l4c_PgkMgrInnit,
//                       added (2) more datasets to the SMAP_L4_C_FPAR_8DAY
//                       theme, for "ndvi" and "ndvi_qc" and also added (3)
//                       dataset definitions to the PRE_PROC_DEFLATE_1KM theme,
//                       for: fpar_clim_dayDDD, ndvi, ndvi_qc, and one for the
//                       file-level info in each theme where datasets are
//                       referenced.
// history             : 2014-05-12T13:37:00,jmg, in l4c_DAILY_hdf5.c, corrected
// duplicate UUID tag gen. history             : 2014-05-09T10:36:00-07:00, in
// l4c_PreProc1km_func.c added READ of
//                       (2) FLOAT32 arrays of 370, "soc_updated_datetime", and
//                       "ft_updated_datetime" to match the extra checkoff
//                       calendars in v5.0.1 SMAP_L4_C_SOC.h5

// history             : v5.0.1-2014-04-30T12:54:00,  for D5.0.1, enabled
// FparClimatology, PRODUCTIONRUN_TAG history             : Summary of planned
// changes, fixes, enhancements for v5.0.1  a) deprecate WRITE pass-thru of
// surface_flags
//                       b) populate bitflags subfields, c) address rootzone SM
//                       scaling, d) fix fpar_qa bit handling, e) change InCell
//                       function, f) implement Fpar Climatology I/O
// history             : v5.0-2014-03-14T16:38:00-07:00,  for D5 delivery
// history             : v4.4.5-2014-03-13T18:54:00, corrected cast of qcScore
// to uint8, from uint16
//                       and corrected bug in QA/carbon_model_bitflag where a
//                       uint8 was being incorrectly memcpy'd into a uint16.
// history             : v4.4.4-2014-03-12T16:57--implemented QA writes of
// carbon_model_qual_pft{1..8} and
//                       QA qa_cnt_pft{1..8} and qa_cnt, and adding qa bitfield
//                       next
// history             : v4.4.3-2014-03-11T15:41, fixed SOC updates, changed
// name of FPAR_8DAY to use underscore
//                       for consistency with theme name since "-" is an illegal
//                       char,and now, we open ALL HDF5 files once early, leave
//                       open, close at session end, to avoid handle-interleave
//                       mixups.
// history             : v4.4.2-2014-03-05T19:45:00 added RIP parse of
// L4_C_GPP_SOURCE (FPAR|NDVI) to migrate
//                       this to a RIP enabled switch and away from using BPLUT
//                       "{TRUE|FALSE}" flags, and corrected SesCtl subtaskIdStr
//                       type (was int,now char *)
// history             : v4.4.1- 2014-03-03T15:42:00-07:00, added the (2)
// missing SMRZ vars into Pre Proc list
//                       to DEFLATE, and removed these from the  AncImagePkgAdt
//                       (ancImagePkgObj) even though they reside in
//                       SMAP_L4_C_Ancillary as datasets, their memory vars are
//                       in MetArrayObj..
// history             : v4.4.0-2014-03-02T21:11:00, attached all 1D sparse
// vectors to their
//                       EcoStateArray and MetArray destination variables.
// history             : v4.3.0-2014-02-27T15:44;00, MILESTONE, pre-processor
// table driven logic working. history             : v4.3.0-2014-02-27T12:21:00,
// re-org PkgInit, split out into sep functions
//                       and added l4c_DAILY_PkgInit_Themes.c to hold all Theme
//                       initializations
// history             : v4.2.9-2014-02-24T15:02-07:00, added qv2m,t2m_avg,ps
// met variables to
//                       tcf_struct.h MetArrayAdt and MetAdt, to accomodate
//                       inputs from SMAP_L4_C_MET.h5
// history             : v4.2.8-2014-02-21T00:25:00-07:00, HDF5 I/O on ancillary
// layers revisted history             : v4.2.7-2014-02-20T14:10:00-07:00,
// runtime testing in progress, and removed
//                       CONT nElem-XX-kmPerTask to use those in SesCtl,
//                       migrated to SesCtl's subtaskId, subtaskIdStr[] .
// history             : v4.2.9-2014-02-25T10:32:00, another pass on
// l4c_GridManager in progress, along with post-proc history             :
// v4.2.6-2014-02-19T23:44:00, continued patching in RunParamAdt history :
// v4.2.5-2014-02-19T17:02:00, created RunParamAdt to split out RIP NVPs, too
// many for SesCtl history             : v4.2.4-2014-02-19T13:20:00, chkpt'd
// work on pre+post processing 'wiring', still in progress history             :
// v4.2.3-2014-02-18T14:48:00, added HDF5 code, and Hdf Manager, and interface
// wrk on mkgrid history             : v4.2.2-2014-02-17T23:51:00, interface wrk
// on mkgrid+ refined HDF5 file creation for dcpls history             :
// v4.2.2-2014-02-17T22:09:00, cut off tcf_inc.h, intrg GLOB_PARAMS into SesCtl
// history             : v4.2.2-2014-02-17T20:09:00, one more pass thru
// tcf_proc9km,mkgrid, on scrubbing
//                       out glob_grid_params
// history             : v4.2.1-2014-02-17T19:45:00-07:00, memory mgmt re-org
// done, working on mkgrid now history             :
// v4.2.0-2014-02-17T10:50:00-07:00, more refactoring on mem+load I/O ops, and
//                       migrated protoypes into tcf_func.h, eventually will
//                       reference all of the TCF specific protos
// history             : v4.1.9-2014-02-16T21:29:00-07:00, large scale code
// re-arrangement, mem allocs and I/O history             :
// v4.1.8-2014-02-13T09:41:00-07:00,jmg -patched in frm_utilGetCtime() history
// : v4.1.7-2014-02-12T18:15:00-07:00,wheat history             :
// v4.1.7-2014-02-05T16:47:00, split out preprocess gridding from
// l4c_InitSession
//                       began re-integration of frm_api, nsb_api, and pkg_mgr
// history             : v4.1.6-2014-01-31T12:46:00-07:00,wheat,jmg, changed EG
// to EGv2 to clarify from EG v1 history             :
// v4.1.6-2014-01-31T12:46:00-07:00,wheat,jmg,prototyped SubtaskAdt,SubTaskMgr
// ADT history             : v4.1.5-2014-01-27T12:12:00-07:00,wheat history :
// 2014-01-19T18:41:00-07:00,wheat,jmg history             :
// 2014-01-16T16:29:00-07:00,wheat,jmg history             :
// 2014-01-14T09:32:00-07:00,wheat,jmg history             :
// 2013-11-25T17:38:00-07:00, re-factoring v1 pass 3 as v1.3
//
// Note      : the tag "L4C_SUBSYSTEM_RELEASE" here refers NOT JUST to a version
// but also includes
//           a release date.  Thus, it refers to NOT JUST software, but also the
//           combination of frozen ANCILLARY DATA (BPLUT,SMAP_L4_C_Ancillary.h5)
//           in addition to just the software itself.
//
// notes:  SOC Calendar rules:
//           0=RESET all soc_calendar slots to 0| 1=Set to 1 all until doy,
//           2=set '1' for JUST this doy
//         FT rules:
//           0=RESET all ft_calendar slots to 0| 1=Set to 1 all until doy, 2=set
//           '1' for JUST this doy
//
// Notes               : enable macro L4_C, and note optional macros: UUID_REQ,
// DIAG_ALGOR. Notes                 in structure CONT, taskIdStr[] was missing,
// Joe added it to allow make process to complete.
//                       rescaled and type-converted all tile_xxx template image
//                       files
// notes    : the NSB database is a name-value-pair stringblock in memory
//            database of NVP's parsed out of the RIP file.
// build    : The following compile snippet shows the baseline configuration for
// a gcc debug build
//            at Univ. Montana NTSG development environment
//            gcc -std=c99 -g -Wall -Wextra -fPIC -D_GNU_SOURCE -DHAVE_UUID
//            -DHAVE_HDF4 -DHAVE_HDF5 -o l4c_DAILY.ex
// ---------------------------------------------------------------

#ifndef l4c_DAILY_DEF
#define l4c_DAILY_DEF 1

// <JMG> declare executable version id with datetime stamp, use across ALL
// components
#define L4C_VERS_TAG "v7.4.3-2021-10-26T13:27"

#define L4C_APP_TAG "l4c_DAILY.ex"

// ---------------------------------------------------------------
// DEPENDENCIES

// Set identifying signatures of major software subcomponents
#define L4_C 1

// define ENABLE_RMSE  1
// this scaled down enable just writes empty HDF5 datasets
#define ENABLE_RMSE_WRITESTUB 1

// LLI's type-independent-vector API
#define TIV_REQ 1

// HDF4|5 _REQ superceded by HAVE_HDF4, HAVE_HDF5
#define HDF5_REQ 1

// Currently these are all now set in the high level makefile
#define HAVE_HDF4 1
#define HAVE_HDF5 1

// Enabled if UUID services present on platform
//   #define HAVE_UUID 1
// Enabled if -lpthread available on platform
// #define HAVE_PTHRD 1

// ----headers----------------------------------------------------
#include <assert.h>
#include <ctype.h>
#include <limits.h>
#include <math.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h> // for getpid()
#include <time.h>
#include <unistd.h> // for gethostname() and getpid()

#ifdef HAVE_UUID
#include <uuid/uuid.h> // for uuid services, uuid_generate() etc..
#endif

// TBD: 2014-08-12, re-verify we can now completely deprecate this older
// way of doing this, then remove this include below.
//     Import default INPUT and OUTPUT HDF5 DATASET names
// #include "l4c_DAILY_DSET_NAMES.h"

// Import TCF model headers
// TCF_SPIN specific headers, leave these called in this order!
#include "l4c_DAILY_tcf_const.h"
// was tcf_struct.h
#include "l4c_DAILY_tcf.h"
#include "l4c_DAILY_tcf_prot.h"
// leave as is for now
#include "uuta.h"

// orig was  "spland_mkgrid.h"
#include "l4c_DAILY_LandMkgrid.h"

// later, merge these into tcf_struct.h and tcf_func.h
// but for now, leave in fpar_qa_bin_util.h...
// orig was "fpar_qa_bin_util.h"
#include "l4c_DAILY_FparQaBin.h"

// Import EASEGRID and related grid constants, was tcf_gridconst.h
#include "l4c_DAILY_grid_const.h"

// Import UM FRAMEWORK API wrapper header
#include "frm_api.h"

// Import header for client API.s of frm_api
#include "nsb_api.h"
// Import header for client API.s of frm_api
#include "pkg_mgr.h"

// import HDF4 and HDFEOS required constants
#ifdef HAVE_HDF4
#include "hdf.h"
#include "mfhdf.h"
// verify if HdfEosDef ref is really required any longer
// #include "HdfEosDef.h"
#endif

// import HDF5
#ifdef HAVE_HDF5
#include "hdf5.h"
#include "hdf5_hl.h"
#endif

// ----------- other manifest constants----------------------------

// borrow definition of PI from math.h if otherwise not defined
#ifndef PI
#define PI M_PI
#endif

// establish how to label session log file as either GMT or Localtime
#define L4_C_USE_GMT 1

// BPLUT (should) carry this UUID in production to satisfy
// version match in production. This is NOT yet strictly enforced
#define L4C_BPLUT_UUID "c7b2b8ac-0007-11e5-8364-db69d6de7648"
#define L4C_BPLUT_N_COLUMNS 44

#define L4C_DOYSTR_MAXLEN 32
#define L4C_HOSTNAME_MAXLEN 256

// SOC,FT calendar slots assigned a '1' for each Day of Year (DOY)
// processed. These are base1 {[1]..[366]} as used, NOT base0
#define L4C_N_CALENDAR_SLOTS 370

// PKGMGR PkgMgr fixed limit of n.l4c_DAILY_land_mkgrid.h of different themes
// Set default n. of PkgMgr themes we'll use.
#define L4C_PKGMGR_THEME_COUNT 20

// for nsbGetValueByKey() calls
#define L4C_NVP_BUFFERLEN 256

// HDF5 related constants

// <JMG> 2016-01-07 macro count of OUTPUT HDF5 datasets
// changed from 47 then 64, now to 73 (0..72)
#define L4C_HDF5_TOTAL_DATASETS 73

#define L4C_HDF5_QA_TOTAL_DATASETS 21

#define L4C_HDF5_DATASET_MAX_NAMELEN 256
#define L4C_HDF5_MAXDIMS 3

// max no. of groups we'll define in one HDF5 file
#define L4C_HDF5_GROUP_MAXCOUNT 16

// #define L4C_MET_MISSING_VALUE     -9999.0
// verify if 255?
#define L4C_FPAR_MISSING_VALUE 254

// <LAJ> 6/5/2014, Need FPAR QA value cutoff for when FPAR climatology is
// applied
#define FPAR_QA_USE_CLIM_THRES                                                 \
  4 // Replace FPAR at this re-classed QA threshold with climatology

// <JMG> added FILL_VALUE, but need to further qualify since many different
// variables require different fill values

// TAKE FILL VALUES DIRECTLY FROM L4_C Product Specification Document
#define FILL_VALUE 254
#define BYTE_FILL_VALUE FILL_VALUE
#define FLOAT_FILL_VALUE -9999.0
// verify how often this OUT_OF_DOMAIN constant is used
#define OUT_OF_DOMAIN_VALUE 253
#define FPAR_FILL_VALUE FILL_VALUE
// 2^15(-1) 32768-1
#define SHORT_FILL_VALUE 32767
// 2^16(-2) 65536-1 = 65534
#define USHORT_FILL_VALUE 65534
// <JMG> placeholder, added for completeness
#define INT32_FILL_VALUE 65534
// <JMG> 2020-07-14, added placeholder, added for completeness
#define INT64_FILL_VALUE 65534
// <JMG> placeholder, added for completeness
#define UINT32_FILL_VALUE 65534

// <LAJ> 7/21/14 : Need arbitrarily high/low starting values for min/max
// calculations
#define STARTING_MIN 9999.0
#define STARTING_MAX -9999.0

// <JMG> 2015-07-29, revise to change default to new 9KM GEOS5 FP TS driven 9km
// freeze thaw source..
// <LAJ> 7/25/14, Flag for using either Radar FT or Tsurf FT from L3_SM_A (1 =
// Use Radar FT; Otherwise use Tsurf FT)
#define USE_RADAR_FT 1

// DEFINE KEY memObj "named" objects
#define L4C_MEMOBJ_PFT_M01land "PFT_M01land"

// --------Enumerated constants ----------------------------------

enum ActionStatusValues {
  ActionSuccess,
  ActionFailure,
  ActionPending,
  ActionStatusUnknown
};

// Adapted to support AMP language style RIP parser classification
enum DatatypeClasses {
  IsClassUndefined,
  IsClassBoolean,
  IsClassInteger,
  IsClassReal,
  IsClassString,
  IsClassStringTable,
  IsClassScalar,
  IsClassList,
  IsClassRange,
  IsClassFilespec,
  IsClassDatetime,
  IsClassDirectory,
  IsClassCustom
};

// For name-value-pairs, define more specific, commonly supported numeric types
// NOTE: Order is important here for fwd,inv projection transform code until
// it is modified to make order unimportant
enum EASEGRIDv2_Types {
  IsEGv2_Global_9km,
  IsEGv2_Global_3km,
  IsEGv2_Global_1km,
  IsEGv2_N_9km,
  IsEGv2_N_3km,
  IsEGv2_N_1km
};

enum EASEGRIDv2_StructureMode {
  IsEGv2_UnknownStructure,
  IsEGv2_Sparse1D,
  IsEGv2_Full2D
};

// used by l4c_GridManager() and friends, to call mkgrid style
// pre (DEFLATE) or post (INFLATE) grid  processing
enum AppPhaseIdentifiers {
  DoPreProcessing,
  DoPostProcessing,
  DoDaily_TCF_Model_Process
};

// enum for sesCtl_p->fparState option switch
// <LAJ> 6/4/2014, Added DoUseFparClim for tracking use of Fpar Climatology,
//    and msngFparFile to indicate a missing or an un-readable Fpar file.
// TBD  Recommend changing corresponding structure field from fparState to
// FparState.
enum GppInputSourceChoices {
  DoUseNDVI,
  DoUseFpar,
  DoUseFparClim,
  msngFparFile
};

// enum indicating which source info used to populate the F/T state
// <JMG> added 7/25/2014 with LAJ. Not yet used but will be soon.
enum FreezeThawInputSourceChoices {
  DoUseFT_ActiveRadar,
  DoUseFT_ActiveTsurf,
  DoUseFT_GEOS5,
  DoUseFT_PassiveRadiom,
  DoUseFT_PassiveTsurf,
  DoUse_NO_FT
};

// Reminder: surface_flag bit {15,15} used to classify if pixel in swath
// or not, where 0= IS IN SWATH and 1=NOT IN SWATH
enum FreezeThawSwathChoices { IsInSwath, IsNotInSwath };

// CF dataset attribute definitions when included in a RIP file:
// This is the enum establishing order of comma-delimited list of a variables
// properties
//  when included in a RIP file's variable declaration. Two forms of
//  INPUT or OUTPUT variable declarations are supported: a simple NVP form,
//  and b) a compound form, whose extended (compound) syntax is illustrated
//  here: L4C_OUTPUT_DATASET =
//  NEE/nee_mean,float32,gc/m2/day,-9999.0,-20.0,100.0,nee_mean
//
enum NsbCompoundPropertyPositions {
  IsPropertyName,
  IsPropertyDatatype,
  IsPropertyUnits,
  IsPropertyFillvalue,
  IsPropertyValidMin,
  IsPropertyValidMax,
  IsPropertyLongname
};

// --------Abstract data types -----------------------------------

// TIV datatypes now in frm_api.h --> frm_tiv_api.h
// reminder: uint32,float32 etc types are all now defined in Framework API

// ---------------------------------------------------------------
// legacy ADTs
// ---------------------------------------------------------------

// ---------------------------------------------------------------
// local ADTs
// -------------------------------------------------------------

// <JMG> 2014-10-06T16:20:00, added missing ISO elements Joe A reported
//
// NEE_COMPLETENESS_OMISSION, GPP_COMPLETENESS_OMISSION,
// RH_COMPLETENESS_OMISSION SOC_COMPLETENESS_OMISSION, EC_COMPLETENESS_OMISSION
// NEE_DOMAIN_CONSISTENCY, GPP_DOMAIN_CONSISTENCY,RH_DOMAIN_CONSISTENCY,
// SOC_DOMAIN_CONSISTENCY, EC_DOMAIN_CONSISTENCY
typedef struct tagL4C_ISO_MetaElements {
  float32 nee_CompletenessOmission;
  float32 gpp_CompletenessOmission;
  float32 rh_CompletenessOmission;
  float32 soc_CompletenessOmission;
  float32 ec_CompletenessOmission;
  float32 nee_DomainConsistency;
  float32 gpp_DomainConsistency;
  float32 rh_DomainConsistency;
  float32 soc_DomainConsistency;
  float32 ec_DomainConsistency;
  // others added later as needed
} L4C_ISO_MetaElements;

// <JMG> 2014-10-16T15:32:00, structure to track frequency of occurance
// of a uint8 var's values over session lifetime
typedef struct tagPostProcFreqVerifyMetrics {
  char pfvm_Uint8Varname[128]; // name of uint8 var analyzed
  size_t pfvm_Uint8freq[256];  // freq counts for uint8 {0...255}
} PostProcFreqVerifyMetrics;

// <JMG> 2014-09-12T12:36, added, to capture observed (non-fillvalue)
//  metrics and bare-bones univar stats on key output variables.
// Note: this object is GenericMetrics in the UM_Framework, serviced
// by frm_utilSummarizePixel()...
typedef struct tagPostProcMetrics {
  float64 obs_min;           // observed minimum (non-fillvalue)
  float64 obs_max;           // observed maximum (non-fillvalue)
  float64 obs_range;         // observed range   (non-fillvalue)
  float64 obs_mean;          // observed mean   (non-fillvalue)
  float64 obs_stddev;        // observed std deviation (non-fillvalue)
  float64 obs_stderror;      // observed std error (non-fillvalue)
  float64 obs_pctstderror;   // observed percent std error (non-fillvalue)
  float64 obs_coef_var;      // observed percent std error (non-fillvalue)
  float64 optionScaleFactor; // optional 'scale-by' factor to transform prior to
                             // output
  // Below are useful when a valid_min,valid_max bracket is available
  float64 minAllow; // optional, value of smallest qualifying pixel value
  float64 maxAllow; // optional, value of largest qualifying pixel value
  size_t obs_CountQualified; // count of non-fillvalue pixels
} PostProcMetrics;

// <JMG> 2014-08-21T19:15:00, tracer on potential gpp or nee issues
typedef struct tagPostInvalid {
  size_t invalidGpp; // count of invalid GPPs
  size_t invalidGppMin;
  size_t invalidGppMax;
  size_t invalidNee; // count of invalid NEEs
  size_t invalidNeeMin;
  size_t invalidNeeMax;
  size_t invalidCountForStddev; // not yet used
  size_t invalidHitNaN;         // added 2014-11-11T13:23 to track NaN counts
  size_t invalidHitInf;         // added 2014-11-11T13:23 to track Inf
  size_t invalidSocMean;        // added 2015-10-15T12:40 to track SOC/soc_mean
                                // setbit(3,3) instances
  float32 qa_CompletenessOmission;
  float32 qa_DomainConsistency;
} PostInvalid;

// Stores separate dcpl's and dspaces for each supported (output)
//  grid resolution combo. 9km needs both a uint8 and a float32 dcpl
//  In general we don't need explicit dcpl's and dspace's for read only
//  datasets.
// <JMG> 2016-01-07: It is very inadvisable practice to hard wire a
// fixed array as originally in dset_L4C_IdAry[], capping the number of output
// HDF5 datasets (e.g. originally 64), so migrate all logic away from this very
// bug-prone approach asap.
// <JMG> 2020-07-14, added dcpl_9km_int64 for hid_t compatibility HDF5 v1.10.6
// and up
//
typedef struct tagHdf5GridPropAdt {
  int easeGridId;       // IsEGv2_Global_1km,IsEGv2_Global_3km,IsEGv2_Global_9km
  hid_t dcpl_1km;       // for 1km
  hid_t dcpl_3km;       // only uint8's needed, for 1 dcpl is sufficient for 3km
  hid_t dcpl_9km;       // for float32 dataset (fillvalue, etc)
  hid_t dcpl_9km_uint8; // for QA uint8   datasets (fillvalue etc)
  hid_t dcpl_9km_int16; // for QA int16   datasets (fillvalue etc)
  hid_t dcpl_9km_uint16; // for QA uint16  datasets (fillvalue etc)
  hid_t dcpl_9km_int32;  // for QA int32   datasets (fillvalue etc)
  hid_t dcpl_9km_int64;  // for QA int64   datasets (fillvalue etc), added
                         // 2020-07-14
  hid_t dcpl_9km_uint32; // for QA uint32  datasets (fillvalue etc)
  hid_t dspace_1km;
  hid_t dspace_3km;
  hid_t dspace_9km;
  // now dset array sized to match no. of actual output datasets, plus 1
  hid_t dset_L4C_IdAry[L4C_HDF5_TOTAL_DATASETS + 1];
  int nHdf5OutputVariableCount;
} Hdf5GridPropAdt;

// <JMG> 2014-11-03, prep for future multiple-day mode, not yet implemented.
// Container for newly reorganized long-lived pre and post processor
//  re-usable buffers, as well as other special-cases, that were previously
//  allocated in l4c_PreProc1km_func.c or
//  in l4c_PostProc9km_func.c.  These are being migrated into this long-lived
//  ADT to facilitate multiple-day processing where redundant alloc,free of
//  these variables is redundant and sloppy.
typedef struct tagPrePostImgPkgAdt {
  size_t nElemPreProcM01;         // EGv2_GLOBAL_1KM_PIXELS
  size_t nElemPost_StateOutput2D; // EGv2_GLOBAL_9KM_PIXELS
  size_t nElemFrozenArea2D;       // EGv2_GLOBAL_9KM_PIXELS
  size_t nElemVPD_M09land;        // EGv2_LDOM9KM, in l4c_PreProc1km_func.c

  // Prior: preProc_M01_image_p in l4c_PreProc1km_func.c
  float32 *preProc_M01_p; // sizeof(float32) * EGv2_GLOBAL_1KM_PIXELS ;
  // Prior: metArrayObj_p->vpd = memCalloc( (size_t)EGv2_LDOM9KM ,
  // sizeof(float32));
  float32 *vpd_M09land_p; // prior: EGv2_LDOM9KM, in l4c_PreProc1km_func.c
  // Prior: stateOutput2D_p , in l4c_PostProc9km_func.c
  float32
      *postProc_stateOutput2D_p; // sizeof(float32) * EGv2_GLOBAL_9KM_PIXELS;
  // Prior: frozenArea_M09_2DFullGrid_p =
  // memCalloc(EGv2_GLOBAL_9KM_PIXELS,sizeof(float32));
  float32 *frozenArea_M09_2D_p;
  // TBD: RMSE specific, and other similar ADTs may be added to this container
  float32 *rmse_mean_M09_p;
  float32 *rmse_pftList_M09_p[NPFT]; // array of (8) pointers to PFT wise RMSE
} PrePostImgPkgAdt;

// Container for SMAP_L4_CA_Ancillary.h5 images
typedef struct tagAncImgPkgAdt {
  size_t nM01land_vecpix; // count of pixels in M01land domain
  size_t nM03land_vecpix; // count of pixels in M03land domain
  size_t nM09land_vecpix; // count of pixels in M09land domain

  uint8 *imgM01land_PixFilter_p; // L4C_M01land_PixFilter_uint8

  // Key static 2D fullgrid images
  uint8 *imgPftM01_dom_p;        // MCD12Q1_M01_lc_dom_uint8
  uint8 *imgM09_lc_mask_uint8_p; // MCD12Q1_M09_lc_mask_uint8

  // Key static 1D sparse vectors
  uint8 *imgPftM01land_dom_p; // MCD12Q1_M01land_lc_dom_uint8

  uint16 *imgM09land_col_uint16_p; // MCD12Q1_M09land_col_uint16
  uint16 *imgM09land_row_uint16_p; // MCD12Q1_M09land_col_uint16
  float32 *imgM09_latitude_img_p;  // SMAP_L4_C_LAT_1624_x_3856_M09_flt32
  float32 *imgM09_longitude_img_p; // SMAP_L4_C_LONT_1624_x_3856_M09_flt32
  // SOC calendar check dailyflags
  // <JMG> 2014-11-11, added cumulative equiv calendar arrays for cross check
  uint8 SOC_CalendarCheck[L4C_N_CALENDAR_SLOTS]; // set 1 in slot 1..366 when
                                                 // day processed
  uint8 FT_CalendarCheck[L4C_N_CALENDAR_SLOTS]; // set 1 in slot 1..366 when day
                                                // processed
  // added cross-check, cumulative form, where for each model run instance
  // we INCREMENT current value of the given day slot instead of assigning a 1
  // to it
  int16 SOC_CalendarCumulative[L4C_N_CALENDAR_SLOTS]; // set 1 in slot 1..366
                                                      // when day processed
  int16 FT_CalendarCumulative[L4C_N_CALENDAR_SLOTS];  // set 1 in slot 1..366
                                                      // when day processed
  // <JMG> 2014-05-09T10:34, added, to match SMAP_L4_C_SOC.h5 at v5.0.1
  float32
      soc_updated_datetime[L4C_N_CALENDAR_SLOTS];    // matches SMAP_L4_C_SOC.h5
                                                     // attribs
  float32 ft_updated_datetime[L4C_N_CALENDAR_SLOTS]; // matches SMAP_L4_C_SOC.h5
                                                     // attribs
} AncImagePkgAdt;

// 2/19/2014, split out bulk of old GLOB_PARAMS into this
//  container, to also contain all other RIP values not relating to
//  direct 'controls' which are kept in SesCtl.
// all these next were in GLOB_PARAMS or CONT. Temp stubs until scrubbed out
typedef struct tagRunParamAdt {
  char subsetName[32];     // default for grid run is "None"
  int fpar_qa_year;        // was in glob_grid_parms
  int nsite;               // was in glob_grid_params
  int spinup_iter_cnt;     // was in glob_grid_params
  float32 spinup_stop_tol; // was in glob_grid_params
  float32 spinup_max_iter; // was in glob_grid_params
  float32 FPAR_scale;      // used to be in BPLUT
  float32 FPAR_offset;     // used to be in BPLUT
  float32 NDVI_scale;      // <JMG> 2014-08-22T01:46:00 added
  float32 NDVI_offset;     // <JMG> 2014-08-22T01:46:00 added
} RunParamAdt;

// <JMG> 2014-11-18, part of SesCtl (orig now too multifaceted) to refactor
// into structures to encapsulate role functionality in separate functions,
// such as: SesPathAdt, SesPkgAdt (holding PkgMgr and Nsb ptr etc)...
typedef struct tagSesPkgPathAdt {
  // PATH related entries
  char inputRipPath[FILENAME_MAX];
  char inputRootDir[FILENAME_MAX];
  char inputAncPath[FILENAME_MAX];
  char inputSOCPath[FILENAME_MAX];
  char inputMetPath[FILENAME_MAX];
  char inputFparPath[FILENAME_MAX];
  char inputFparClimatologyPath[FILENAME_MAX];
  char inputMetClimatology[FILENAME_MAX];
  char inputBplutPath[FILENAME_MAX];
  // OUTPUT related entries
  char outputRootDir[FILENAME_MAX];
  char output_L4C_DAILY_Path[FILENAME_MAX];
  // LOG related entries
  char logPath[FILENAME_MAX];
  char logRootDir[FILENAME_MAX];
  char logInitialPath[FILENAME_MAX];
  // final log path resolved only at end of session
  char logFinalPath[FILENAME_MAX];
} SesPkgPathAdt;

// <JMG> 2014-11-18, split out from SesCtl into narrowly focused ADT
// containing just the HDF5 file id's
typedef struct tagSesPkgHdf5IdAdt {
  // HDF5 file id's will be split into into SesPkgHdf5FileIdAdt
  hid_t anc_file_id;       // HDF5 file_id of SMAP_L4_C_Ancillary.55
  hid_t met_file_id;       // HDF5 file_id of SMAP_L4_C_MET.h5
  hid_t soc_file_id;       // HDF5 file_id of SMAP_L4_C_SOC.h5
  hid_t soc_group_id;      // required for some accesses...
  hid_t fpar_file_id;      // HDF5 file_id of SMAP_L4_C_MODIS.h5 or VIIRs
  hid_t l4cOutput_file_id; // HDF5 file_id of SMAP_L4_C_OUTPUT.h5
  hid_t fparClima_file_id;
  hid_t viirs_file_id; // VIIRs NDVI now in same SMAP_L4_C_FPAR_8DAY.h5 file
} SesPkgHdf5IdAdt;

// NOTE: removed from SesCtl, too much...
//  Hdf5GridPropAdt h5GridPropObj; // HDF5 dspace,dcpl's by resolution;

// master session controls object, implemented in: l4c_ses.c
// 2/19/2014, refactored to KEEP just the real "controls" and
// place all other runtime parameters in L4C_RunParams --> l4cRunParamObj

typedef struct tagSesCtl {
  int runDayOfYear;       // day of year this execution session is running
  int runToStopDay;       // 2014-11-03,now 'runToStopDay' was 'dayOfYearToStop'
  int runDayStepInterval; // 2014-11-03,added for future multi-day capability
  int runCompositePeriod; // FPAR 8-day composite period index
  int runModelYear;       // YYYY assoc with this model day run
  int sesNvpCount;        // count of NVP's parsed from RIP file
  int dayOfYearToStop;    // replaces CONT doyStep now deprecated
  int hdf5ErrStackState; // <JMG> 6/17/2014, stores 0 if stack off,1 if stack on

  // SPATIAL-STEP-INTERVAL Default=1, if >1 allows for systematic skip-sampling
  size_t runPixStepInterval;
  // Prior, in ADT "CONT". TBD: Will need updating when we re-implement multiple
  // task mode!
  int nSubtasks;         // was in CONT, as nTasks
  int subtaskId;         // was in CONT. as 'taskId'
  char subtaskIdStr[32]; // was in CONT, as 'taskIdStr'

  // TBD: migrate these counts to size_t types for consistency
  int nElem9kmPerTask; // for array-parallel proc, was in CONT control_struct
  int nElem3kmPerTask; // was in CONT control_struct
  int nElem1kmPerTask; // was in CONT control_struct
  int elem9kmOffset;   // for data-parallel by array processing, TBD
  int elem3kmOffset;
  int elem1kmOffset;

  int enableRMSE;      // boolean, 1=enables output of all QA/xx_rmse variables
  int enableSOCUpdate; // boolean, 1=enable SOC update, 0=disable update
  int enableGranuleMetrics; // L4C_GRANULE_METRICS boolean 1=enable 0=disable
  int enableCompleteMsg;    // boolean, 1=enable COMPLETION msg 0=suppress
  int enableWriteObsMinMax; // boolean, 1=write out obs min,max,mean for each
                            // dataset
  int compressLev;          // default 6 or 7 out of {1..9}

  // <JMG> 2014-07-25T17:27, changed 'gppFpar_or_NDVI' to fparState.
  int fparState; // option: DoUseFpar,DoUseNVDI ..switch

  // <JMG> 2015-0-29T10:58, extended to support new GEOS5 FP TS derived 9KM F/T
  // <JMG> 2014-07-25T17:56, added to catch F/T similar to fparState works
  // DoUseFT_ActiveRadar, DoUseFT_ActiveTsurf,
  // DoUseFT_PassiveRadiom,DoUseFT_PassiveTsurf, DoUseFT_GEOS5
  int ftState;

  // 0=RESET all soc_calendar slots to 0| 1=Set to 1 all until doy, 2=set '1'
  // for JUST this doy
  int socOpMode;
  // 0=RESET all ft_calendar slots to 0| 1=Set to 1 all until doy, 2=set '1' for
  // JUST this doy
  int ftOpMode;

  // was "action" for, "step_forward" etc from RIP file
  char appAction[32]; // 'production'| 'development'|'local' etc
  char
      gppMethodStr[16]; // <JMG> 2014-08-21, added to match l4c_MODIS convention
  char fparSourceStr[6]; // <KAE> 2021-11-01 To contain "VIIRS" or "MODIS"
  char thisHost[L4C_HOSTNAME_MAXLEN]; // hostname we're running on
  char thisUsername[256];             // username controlling process' terminal
  char productionLevel[32];           // set context as "local"|"production"
  char doi_assigned[32];   // L4CSYS std, only L4CMDL has a DOI so far
  char ftMethodString[16]; // also stores FT Method string for clarity

  // COMMON VERSIONING objects
  char SubsysReleaseTag[128]; // shared subsys RELEASE identifier
  char swVersion[64];         // like: "v5.0.1", as L4_C_SW_VERSION
  char swRelease[64]; // like: "v5.0.1-2014-09-10T16:44:00" as L4_C_SW_RELEASE
  // char  **ripTbl_p ; // ptr to master RIP NVP string table
  char l4c_Task_Label[256]; // production run identifier
  char l4c_Task_UUID[64];   // from RIP file NVP named L4C_TASK_UUID
  // <JMG> 2017-06-05 added for soft BPLUT version check at runtime
  char l4c_BPLUT_UUID[64]; // from RIP file NVP named L4C_BPLUT_UUID
  char l4c_BPLUT_MD5[64];  // from RIP file NVP named L4C_BPLUT_MD5
  char l4c_Platform[128];  // L4C_PLATFORM ..name of (host) running this job
  char granuleUUID[64];    // only uses 37 bytes of this for UUID
  char ScienceVersID[64];  // new, match Rolf's L4_SM output name scheme
  char CRID[64];           // legacy CRID may not be needed?
  char frmApiVersion[64];  // UM Framework version
  char pkgApiVersion[64];  // PkgMgr version
  char nsbApiVersion[64];  // NSB version
  char datasetVersion[64]; // L4C_DATASET_VERSION
  // governing ATBD and PSD attributes
  char govATBD_Vers[16];
  char govATBD_Date[16];
  char govPSD_Edition[16];
  char govPSD_Date[16];

  // COMMON temporal session state terms
  char doy_Str[8];      // day-of-year string
  char year_Str[8];     // YEAR YYYY string, year pertaining to this run
  char period_Str[8];   // 8-day composite period in string form {01..46}
  char period16_Str[8]; // 16-day composite period in string form {01..23}

  // PATH related entries
  char inputRipPath[FILENAME_MAX];
  char inputRootDir[FILENAME_MAX];
  char inputAncPath[FILENAME_MAX];
  char inputSOCPath[FILENAME_MAX];
  char inputMetPath[FILENAME_MAX];
  char inputFparPath[FILENAME_MAX];
  char inputFparClimatologyPath[FILENAME_MAX];
  char inputMetClimatology[FILENAME_MAX];
  char inputBplutPath[FILENAME_MAX];
  // OUTPUT related entries
  char outputRootDir[FILENAME_MAX];
  char output_L4C_DAILY_Path[FILENAME_MAX];

  // <KAE> 2024-01-03 For litterfall allocation scheme
  char litterfallScheme[32];
  char litterfallFileTemplate[FILENAME_MAX];

  // LOG related entries
  char logPath[FILENAME_MAX];
  char logRootDir[FILENAME_MAX];
  char logInitialPath[FILENAME_MAX];
  // final log path resolved only at end of session
  char logFinalPath[FILENAME_MAX];
  FILE *logf; // ptr to session log file handle

  // <JMG> added 4/22/2014 to allow l4c_SesInit to
  // dynamically assign based on runDayOfYear,runCompositePeriod
  char fparClimDatasetname[64];
  // ANCILLARY dataset related entries (added 9/4/2014)
  char latCoordVarname[128];
  char lonCoordVarname[128];

  // session metrics
  time_t sesStrtSec; // session starting time, in sec
  time_t sesEndiSec; // session ending time, in sec
  double sesElapSec; // elapsed seconds for session

  // HDF5 file id's will be split into into SesPkgHdf5FileIdAdt
  hid_t anc_file_id;       // HDF5 file_id of SMAP_L4_C_Ancillary.55
  hid_t met_file_id;       // HDF5 file_id of SMAP_L4_C_MET.h5
  hid_t soc_file_id;       // HDF5 file_id of SMAP_L4_C_SOC.h5
  hid_t soc_group_id;      // required for some accesses...
  hid_t fpar_file_id;      // HDF5 file_id of SMAP_L4_C_MODIS.h5 or VIIRs
  hid_t l4cOutput_file_id; // HDF5 file_id of SMAP_L4_C_OUTPUT.h5
  // <JMG> deprecated ".qa" file entirely on merge of QA data into "mdl" .h5
  // file hid_t  l4cQa_file_id;     DEPRECATED -- HDF5 file_id of
  // SMAP_L4_C_OUTPUT.qa
  hid_t fparClima_file_id;
  hid_t viirs_file_id; // VIIRs NDVI now in same SMAP_L4_C_FPAR_8DAY.h5 file

  // 2/19/2014, Replacement ATD for GLOB_PARAMS and also, contains all other RIP
  // entries too numerous to dump into SesCtl... <JMG> converted to auto scalar,
  // not heap allocated now
  RunParamAdt runParamObj; // scalar of (legacy) L4_C runtime parameters

  NsbObj *sesNsbDb_p;  // ptr to NVP stringblock database OVERSEER mgr
  PkgMgr *l4cPkgMgr_p; // ptr to PkgMgr OVERSEER mgr
  int nPkgThemes; // count of PkgMgr themes we'll use thru session life-cycle

} SesCtl;

#endif
