// ------------------------------------------------------
// module       : l4c_DAILY_LandMkgrid.h

// purpose      : header (renamed) for tcfPreProc_mkgridManager() and friends
//                as the high level entry point for performing the mkgrid 
//                pre-processing which INFLATES or DEFLATES 
//                {1km,3km,9km} EASEGRIDv2 global grids {M01,M03,M09}

//  project      : SMAP L4C science development.
//  dependencies : HDF4, FRM_API etc
//  revised      : 2014-02-15T17:21:00-07:00,jmg, LEAD FORK PASS 3 integration
//  author       : joe glassy, Lupine Logic Inc.
//  history      : Initial reformating -- laj 09/10/13
//  author       : Lucas A. Jones; lucas@ntsg.umt.edu
//  notes        : Uses module-static log handle which is thread-unsafe; eventually will
//                 migrate to use thread-safe frm_log_mgr() services.
//  notes        :  PRIOR: CtrlStruct   NEW NS: CtrlProc9kmAdt


#ifndef SPARSE_LAND_DEF
#define SPARSE_LAND_DEF 1

#include <stdio.h>
#include <stdlib.h>
// size_t is usually defined here in unistd.h
#include <unistd.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <time.h>
#include <limits.h>
#include <assert.h>

#include  "hdf.h"
#include  "mfhdf.h"

// <JMG> 2/15/2014, add visibility to all of UM Framework 
#include "frm_api.h"

// needed for:  parse_fpath, dtype_from_dtypestr, string_parse, and insert_str
# include "l4c_DAILY_StringUtilTcf.h"

// <JMG> 3/8/2014 MOVED include of uuta out of this, and ahead...
// #include "uuta.h"

// <JMG> WARNING: hard coded paths.. change to RIP entity references
// #define LAND_R_FILE  "/s_measures/laj/landdomdef/output/MCD12Q1_M09land_row.uint16"
// #define LAND_C_FILE  "/s_measures/laj/landdomdef/output/MCD12Q1_M09land_col.uint16"
#define LAND_R_FILE  "USE-RIP-FILE-MCD12Q1_M09land_row.uint16"
#define LAND_C_FILE  "USE-RIP-FILE-MCD12Q1_M09land_col.uint16"

// NOTE: grid constants like LDOM1KM, LDOM3KM,LDOM9KM, NESTED_3KM_IN_9km ARE ALL NOW
// defined in tcf_const.h

// <JMG> TBD, in many cases for the use of this macro as a FILE PATH, it really should be FILENAME_MAX, not 500
// since FILENAME_MAX is a POSIX constant much larger...
#define   MXSTRLEN                 500
// <JMG> TBD, check for name conflict in tcf-spin's tcf_const
#define   FILLVAL                -9999       //  Fill values

// MKGRID defines these same-named macros a little differently so had to
//  rename them to avoid a conflict, and, need to set any calls to these
//  to use the MKGRID_2D_xxx form of these, and NOT the M_2D_xxx form.

// SERIALIZATION OFFSETS 
// 2D offsets: y is outermost, x is innermost (fastest varying) term
#define MKGRID_2D_B0(x,y,n_y) (((x)*n_y)+(y))

//  NEW 6-term Serialization offset macros, for nested grid within 9km 2D grid
//  indices x,y correspond to 9km grid, while i,j is the nested cell reference
#define MKGRID_2D_N9_B0(x,y,i,j,nj, ny) (((x)*nj + (i))*ny + (y)*nj + (j))

// ------------------------------------------------------
// ABSTRACT DATA TYPEs for mkgrid:
// ------------------------------------------------------

// PRIOR : spland_ref_struct   NEW NS: SplandRefAdt
typedef struct tagSplandRefAdt {
  uint16   *row;
  uint16   *col;
} SplandRefAdt; // was: spland_ref_struct


// ------------------------------------------------------
// FUNCTION PROTOTYPES for mkgrid:
// ------------------------------------------------------

int spland_load_9km_rc(SplandRefAdt *SPLAND);

void spland_deflate_driver(char outpath[], char ifpath[]);
void spland_deflate_9km(SplandRefAdt SPLAND, void *src_p, void *dest_p, const uint32 dataType);
void spland_deflate_3km(SplandRefAdt SPLAND, void *src_p, void *dest_p, const uint32 dataType);
void spland_deflate_1km(SplandRefAdt SPLAND, void *src_p, void *dest_p, const uint32 dataType);

void spland_inflate_driver(char outpath[], char ifpath[]);
void spland_inflate_9km(SplandRefAdt SPLAND, void *src_p, void *dest_p, const uint32 dataType);
void spland_inflate_3km(SplandRefAdt SPLAND, void *src_p, void *dest_p, const uint32 dataType);
void spland_inflate_1km(SplandRefAdt SPLAND, void *src_p, void *dest_p, const uint32 dataType);

void spland_inflate_init_9km(void *dest_p, const uint32 dataType);
void spland_inflate_init_3km(void *dest_p, const uint32 dataType);
void spland_inflate_init_1km(void *dest_p, const uint32 dataType);

void set_fillval_UUTA(void *vDest_p,const int32 dataType,const size_t atSlot);


#endif
