# -----------------------------------------------------------
# file         : l4c_DAILY_NCCS.mak
#
# purpose      : Makefile for l4c_DAILY.ex executable, that also now 
#                freshens the underlying UM Framework library build as necessary
# revision    : 2016-02-18T08:51:00, updated for modules renamed in v7.1.0 RMSE enabled
# revision    : 2014-10-01T17:53, updated for recent improvements
# history     : 2014-08-15T13:45:00-07:00, added l4c_DAILY_PrepOutput.c module to compile
# history     : 2014-08-11T18:18:00-07:00, revised for NCCS use
# history     : 2014-07-25T17:13, merged in all recent LAJ and JMG changes and
#                split out RIP nvp logic into l4c_DAILY_rip.c module file.
# history     : 2014-06-24T12:33:00, added l4c_DAILY_WriteMetadata.c to 
#                address need for writing "group-of-groups" metadata
# history      : 2014-05-19T12:28:00, consolidated l4c_DAILY_PkgMgrInit_Themes.c into
#                l4c_DAILY_PkgMgrInit.c to simplify.
# history      : 2014-03-14T15:48:000, modified to also freshen the underlying UM Framework API 
#                library, static  "libfrm.a"
# history      : 2014-03-03T15:42:00, refactored pre proc code into
#                l4c_PreProc1km_Driver.c and added this to Makefile.
# history      : 2014-02-21T15:25:00, further runtimetesting
# history      : 2014-02-20T10:10,now runtime testing, working through early segfaults
# history      : 2014-02-17T20:29:00,improved Makefile from missouri clean ups
# history      : 2014-02-16T16:02:00, lead fork, final integration
# history      : 2014-02-15T16:02:00, lead fork PASS 1,2,3
# history      : 2014-02-14T13:43:00, lead fork review
# history      : 2014-02-05T17:24:00 
# history      : 2013-12-16T16:57:00-07:00,wheat,jmg
# history      : 2013-11-11T09:45:00-07:00,ruby,jmg
# author       : joe glassy, Lupine Logic Inc.
#
# platform     : NCCS Discover and friends
# dependencies : HDF4, HDF5 and FRM_API (libfrm.a) libraries and the following
#                predefined environment variables: FRM_API_ROOT, FRM_API_LIB, FRM_API_INC
#                HDF4_INC,HDF4_LIB, HDF5_INC,HDF5_LIB
#
# dependencies : For builds at Univ Montana, make sure to define FRM_API_ROOT, on stout, as :
#                FRM_API_ROOT=/pilv2/SMAP/L4CSYS/L4CAPI
#
# notes        : The following naming conventions apply:
#			   : --module files starting with "l4c_" indicate they are part
#              :   of the overall L4_C namespace, not particular to just the
#              :   l4c_DAILY code set.
#              : --module files starting with l4c_Day_" indicate they are
#              : specifically part of just the l4c_Day_* algorithm code set.
# -----------------------------------------------------------

BASEDIR = /discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.4_build1/x86_64-unknown-linux-gnu/ifort/Linux
FRM_API_ROOT = /gpfsm/dnb02/projects/p51/L4_C_staging/L4CSYS/v7/L4CAPI

HDF4INC = $(BASEDIR)/include/hdf
HDF4LIB = $(BASEDIR)/lib

HDF5INC = $(BASEDIR)/include/hdf5
HDF5LIB = $(BASEDIR)/lib

CC = gcc

# CFLAGS =  -g -Wall -std=c99 -fPIC -D_GNU_SOURCE -DHAVE_UUID 
# for now remove -DHAVE_UUID
CFLAGS =  -g -Wall -Wextra -fPIC -D_GNU_SOURCE -DHAVE_HDF4 -DHAVE_HDF5 -DHAVE_UUID

# <JMG> 2014-09-30, changed to use FRM_API_BASE on /pilv2/SMAP instead of pilv1
FRM_API_SRC = $(L4CAPI_ROOT)/src
# in case these aren't explicitly defined we define these here:
FRM_API_INC = $(L4CAPI_ROOT)/inc
FRM_API_LIB = $(L4CAPI_ROOT)/lib

INCDIR =.
LIBDIR =.

INCS = -I$(FRM_API_INC) -I$(HDF4INC) -I$(HDF5INC) -I$(INCDIR) 

LIBS = -L$(LIBDIR) -L$(HDF4LIB) -L$(HDF5LIB) -L$(FRM_API_LIB) 

# Note: the order shown for -lhdf5_hl and -lhdf5 is important here to preserve on NCCS!
# LDFLAGS = -lfrm -ldf -lmfhdf -lhdf5_hl -lhdf5 -lz -luuid -lm
# successful syntax from l4c_MET was:
LDFLAGS = -lfrm -ldf -lmfhdf -lhdf5_hl -lhdf5 -ljpeg -lz -lsz -luuid -lm

all: $(FRM_LIB)  $(TARGET) 

# added...
ARTOOL  = ar
ARFLAGS = rv
# added...
.SUFFIXES : .o .c .a .h

# target for building UM Framework API library
FRM_LIB = $(FRM_API_LIB)/libfrm.a

# target for building L4_C executable itself
TARGET = l4c_DAILY.ex


SRCS = l4c_DAILY_Action.c\
	l4c_DAILY_BindToPkgMgr.c\
	l4c_DAILY_bplut.c\
	l4c_DAILY_DailyDriver.c\
	l4c_DAILY_DatasetProperties.c\
	l4c_DAILY_Diagnostics.c\
	l4c_DAILY_EASEGRID_Transforms.c\
	l4c_DAILY_ForwardStep.c\
	l4c_DAILY_FparQaBin.c\
	l4c_DAILY_GridPost_Inflate.c\
	l4c_DAILY_GridPre_Deflate.c\
	l4c_DAILY_hdf5.c\
	l4c_DAILY_Hdf5_Read_and_Updates.c\
	l4c_DAILY_ModFunc.c \
	l4c_DAILY_ModMult.c \
	l4c_DAILY_PkgMgrInit.c\
	l4c_DAILY_PostProcessor.c\
	l4c_DAILY_PrepOutput.c \
	l4c_DAILY_PreProcessor.c\
	l4c_DAILY_rip.c \
	l4c_DAILY_sci.c \
	l4c_DAILY_ses.c \
	l4c_DAILY_StringUtilTcf.c\
	l4c_DAILY_tcfMem.c \
	l4c_DAILY_tcfModDriverSubset.c\
	l4c_DAILY_tcfModQc.c\
	l4c_DAILY_WriteMetadata.c \
	l4c_DAILY.c

SRCS_LIB = $(FRM_API_SRC)/frm_core.c \
	$(FRM_API_SRC)/frm_mem_obj.c \
	$(FRM_API_SRC)/frm_log_msg.c \
	$(FRM_API_SRC)/frm_str_fx.c  \
	$(FRM_API_SRC)/frm_tiv_api.c \
	$(FRM_API_SRC)/frm_io_fx.c \
	$(FRM_API_SRC)/frm_util_fx.c \
	$(FRM_API_SRC)/frm_hdf_io.c\
	$(FRM_API_SRC)/nsb_api.c \
	$(FRM_API_SRC)/nsb_dbsys.c\
	$(FRM_API_SRC)/nsb_accessors.c \
	$(FRM_API_SRC)/nsb_parsers.c \
	$(FRM_API_SRC)/pkg_mgr_objects.c \
	$(FRM_API_SRC)/pkg_mgr_themes.c \
	$(FRM_API_SRC)/pkg_mgr.c 

# Create one object file per .c source file, from the application sources
OBJS = $(SRCS:.c=.o)

# Create one object file per .c source file from the library c sources
# OBJS_LIB = $(SRCS_LIB:.c=.o)

# Global default rule, update the library, then the executable application
all: $(FRM_LIB) $(TARGET)  

# rule for building the application
$(TARGET): $(OBJS)
	$(CC) $(CFLAGS) -o $(TARGET) $(OBJS) $(INCS) $(LIBS) $(LDFLAGS)

# Rule for building or updating the UM Framework library
# from each .o combine to construct the static library, .a
# $(FRM_LIB): ${OBJS_LIB}
# $(ARTOOL) $(ARFLAGS) $@ $?


# Rule to update the .a library whenever a newer .c file is encountered
.c.a :
	$(CC) $(CFLAGS) $(ADDFLAGS) $(INCS) $<
	$(ARTOOL) $(ARFLAGS) $@ $*.o
	rm -f $*.o

# Rule to update the .a library whenever a newer .o file is found
.o.a :
	$(ARTOOL) $(ARFLAGS) $<

clean:
	rm -f *.o *~ *.ex  


install:
	cp $(TARGET) ..


# L4_C standard test stanzas...
# omit for now: l4c_DAILY_VALID.keys
test:
	./$(TARGET) ../../../RIP/l4c_DAILY_v5.0.1.rip 
	less l4c_DAILY.log 


# Update the .o files whenever a newer .c file is encountered
.c.o:
	$(CC) $(CFLAGS) $(INCS) -c $< -o $@

