// ---------------------------------------------------------------
// function : l4c_DAILY_PrepOutput.c
// purpose  : Populate metadata properties of each OUTPUT HDF5 datasets
// inputs   : PkgMgr *pkgMgr_p, char **dsetNameTable_p,const int nPostProcVariables
// revision : 2016-02-18, changed all EC names, mult_av to mult_mean 
// revision : 2014-08-14T15:23:00, re-factored from l4c_GridPost_Inflate.c where
//            these functions as evolved do NOT now belong.
// notes    : This module file created 2014-08-14, added to makefiles

#include "l4c_DAILY.h"
#include "l4c_DAILY_prot.h"


// ---------------------------------------------------------------
// function : l4c_PrepOutput_HDF5_Datasets()
// purpose  : populate metadata properties of each OUTPUT HDF5 datasets
// inputs   : PkgMgr *pkgMgr_p, char **dsetNameTable_p,const int nPostProcVariables
// returns  : int, status
// revised  : 2014-05-19, updated to migrate towards a list driven process
//
// notes    : called l4c_DAILY_ses.c l4c_InitSessions() currently.
//
// notes    : While these properties are hard-coded for now, ideally we
//           want to migrate these to a table driven process,
//           driven of the NSB RIP entries, or other config file etc
// notes    : we refer here to the THEME declarations made in 
//            l4c_DAILY_PkgMgrInit_Themes.c
//
// notes    : EGv2_GLOBAL_1KM_NS_PIXELS      14616
//            EGv2_GLOBAL_1KM_EW_PIXELS      34704   
//            EGv2_GLOBAL_1KM_PIXELS     507233664 
//
int l4c_PrepOutput_HDF5_Datasets(void)
{
 SesCtl   *sesCtl_p = l4c_GetSesCtl();
 PkgMgr   *pkgMgr_p = l4c_GetPkgMgrOverseer();
 PkgTheme *pThme_p = NULL;
 PkgObj   *pObj_p  = NULL;
 char *datasetName_p = NULL;
 char nameBuffer[256] = {0};
 char workBuffer[128] = {0} ;
 char fxName[256] = {0} ;
 int idx =0;
 int nPkgObj = 0 ;
 int p = 0 ;
 int startIdx = -1;
 int endingIdx= -1;
 
    strcpy(fxName,__func__);
    
    // ---------------------------------------------------------------
    // STEP 1: initailize all primary L4_C dataset properties
    pThme_p  = pkgMgr_GetThemeByName(pkgMgr_p,"L4C_OUTPUTS");

    nPkgObj = pkgMgr_GetThemeObjectCount(pThme_p);
    xLogMsg("@A In(%s) Preparing HDF5 OUTPUT (%d PkgObjs)  Dataset Properties~ \n",
       fxName, nPkgObj);

    // FIRST HDF5 dataset named in theme, L4C_OUTPUTS is..
    //  pObj_p   = pkgMgr_GetObjByName(pThme_p,"NEE/nee_mean");
    // LAST HDF5 dataset named in theme , L4C_OUTPUTS is..
    //   pObj_p   = pkgMgr_GetObjByName(pThme_p,"GEO/longitude");
    
    // So now, fetch full range of PkgObj indices that the L4C_OUTPUT theme owns
    //  then auto-iterate through these, setting default properties for all
    //  PkgObj (L4_C output HDF5 datasets,variables) that SHARE the same properties...
    startIdx = pkgMgr_GetObjIndexByName(pThme_p,"NEE/nee_mean");
    endingIdx= pkgMgr_GetObjIndexByName(pThme_p,"GEO/longitude");
    
    xLogMsg("@A In(%s) PkgMgr Output Theme Indices (nee_mean start %d) (longitude end %d)~ \n",
       fxName, startIdx, endingIdx);
    
    // 'idx' indexes into this theme's child PkgObj, one per OUTPUT variable
    for(idx= startIdx; idx < endingIdx; idx++)
    {
        pObj_p = pkgMgr_GetObjByIndex(pThme_p,idx);
        
        datasetName_p = pkgMgr_GetNameByIndex(pThme_p,idx,nameBuffer);
        
        // new usage takes advantage of specific 'path' property
        pkgMgr_SetObjGenericPath(pObj_p, sesCtl_p->output_L4C_DAILY_Path );

        // Load output file_id as 'owner'
        pkgMgr_SetObjOwner(pObj_p, sesCtl_p->l4cOutput_file_id);
        // must later convert datatype to a H5T_NATIVE_FLOAT style identifier
        pkgMgr_SetObjDatatype(pObj_p, (int)DFNT_FLOAT32 );  
            
        pkgMgr_SetObjResolution(pObj_p, 9 ); // output res is 9km not 1km
        pkgMgr_SetObjRank(pObj_p, 2 );
        pkgMgr_SetObjNelem(pObj_p, EGv2_GLOBAL_1KM_PIXELS );
        
        // CF style attributes, set fallback defaults at least
        // <JMG> 2017-07-24 setting un-equivocably to FLOAT_FILL_VALUE
        // is questionable since datasets with integer type may eventually
        // REQUIRE a matched-type fill value. This hasn't been an issue
        // yet but could eventually be. For now we depend on user type-casting it
        // to final correct type on retrieval 
        pkgMgr_SetObjFillvalue(pObj_p, FLOAT_FILL_VALUE);
        pkgMgr_SetObjUnits(pObj_p    , "units-TBD");
        pkgMgr_SetObjValidMin(pObj_p , -123456789.0); // placeholder only 
        pkgMgr_SetObjValidMax(pObj_p , 123456789.0); // placeholder only 
        pkgMgr_SetObjShortname(pObj_p, datasetName_p);// placeholder
        pkgMgr_SetObjLongname(pObj_p , datasetName_p);// placeholder
        
    } // end:: idx loop interating through ALL HDF5 MAIN DATASET definitions 
    
    // Having set the defaults, now go back and override for the exceptions
    // for L4_C datasets whose properties differ from the "majority"
    // Having set the defaults, now go back, SET any overridden exceptions
    
    pObj_p   = pkgMgr_GetObjByName(pThme_p,"QA/carbon_model_bitflag");
    pkgMgr_SetObjGenericPath(pObj_p, sesCtl_p->output_L4C_DAILY_Path);
    pkgMgr_SetObjDatatype(pObj_p, (int)DFNT_UINT16 );
    pkgMgr_SetObjFillvalue(pObj_p, USHORT_FILL_VALUE);

    // <JMG> 2016-01-07, Check missing references for qa_count,carbon_model_qual series 
    for(p=0; p < 8; p++)
    {
      sprintf(workBuffer,"QA/carbon_model_qual_pft%d",p+1);
      pObj_p = pkgMgr_GetObjByName(pThme_p, workBuffer);
      pkgMgr_SetObjGenericPath(pObj_p, sesCtl_p->output_L4C_DAILY_Path);
      pkgMgr_SetObjOwner(pObj_p      , sesCtl_p->l4cOutput_file_id);
      pkgMgr_SetObjDatatype(pObj_p   , (int)DFNT_UINT8 );
      pkgMgr_SetObjFillvalue(pObj_p  , BYTE_FILL_VALUE);
      pkgMgr_SetObjResolution(pObj_p , 9 ); // output res is 9km not 1km
      pkgMgr_SetObjRank(pObj_p       , 2 );
      pkgMgr_SetObjNelem(pObj_p, EGv2_GLOBAL_9KM_PIXELS );      
      pkgMgr_SetObjValidMin(pObj_p , 0)  ; // placeholder only 
      pkgMgr_SetObjValidMax(pObj_p , 100); // placeholder only 
    } // end:: p iterate through (8) PFT classes
    
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GEO/latitude");
    pkgMgr_SetObjDatatype(pObj_p,  DFNT_FLOAT32 );
    pkgMgr_SetObjFillvalue(pObj_p, FLOAT_FILL_VALUE);
    
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GEO/longitude");
    pkgMgr_SetObjDatatype(pObj_p,  DFNT_FLOAT32 );  
    pkgMgr_SetObjFillvalue(pObj_p, FLOAT_FILL_VALUE);
    
    // ---------------------------------------------------------------
#ifdef FOR_REFERENCE_ONLY
    pThme_p  = pkgMgr_GetThemeByName(pkgMgr_p,"L4C_OUTPUTS"); // THEME: "L4C_OUTPUTS" 

    pObj_p = pkgMgr_GetObjByName(pThme_p, "NEE/nee_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "NEE/nee_std_dev");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "NEE/nee_pft1_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "NEE/nee_pft2_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "NEE/nee_pft3_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "NEE/nee_pft4_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "NEE/nee_pft5_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "NEE/nee_pft6_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "NEE/nee_pft7_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "NEE/nee_pft8_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GPP/gpp_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GPP/gpp_std_dev");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GPP/gpp_pft1_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GPP/gpp_pft2_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GPP/gpp_pft3_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GPP/gpp_pft4_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GPP/gpp_pft5_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GPP/gpp_pft6_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GPP/gpp_pft7_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GPP/gpp_pft8_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "RH/rh_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "RH/rh_std_dev");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "RH/rh_pft1_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "RH/rh_pft2_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "RH/rh_pft3_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "RH/rh_pft4_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "RH/rh_pft5_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "RH/rh_pft6_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "RH/rh_pft7_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "RH/rh_pft8_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "SOC/soc_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "SOC/soc_std_dev");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "SOC/soc_pft1_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "SOC/soc_pft2_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "SOC/soc_pft3_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "SOC/soc_pft4_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "SOC/soc_pft5_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "SOC/soc_pft6_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "SOC/soc_pft7_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "SOC/soc_pft8_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "EC/frozen_area");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "EC/emult_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "EC/tmult_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "EC/wmult_mean");

    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/carbon_model_bitflag");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/carbon_model_qual_pft1");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/carbon_model_qual_pft2");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/carbon_model_qual_pft3");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/carbon_model_qual_pft4");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/carbon_model_qual_pft5");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/carbon_model_qual_pft6");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/carbon_model_qual_pft7");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/carbon_model_qual_pft8");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/qa_count_pft1");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/qa_count_pft2");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/qa_count_pft3");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/qa_count_pft4");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/qa_count_pft5");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/qa_count_pft6");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/qa_count_pft7");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/qa_count_pft8");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/qa_count");
    
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/nee_rsme_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/nee_rsme_pft1_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/nee_rsme_pft2_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/nee_rsme_pft3_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/nee_rsme_pft4_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/nee_rsme_pft5_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/nee_rsme_pft6_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/nee_rsme_pft7_mean");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "QA/nee_rsme_pft8_mean");
    
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GEO/latitude");
    pObj_p = pkgMgr_GetObjByName(pThme_p, "GEO/longitude");
#endif  
    
    l4c_EventCompletionMsg((char *)__func__);   
    
 return 0;
    
} // end:: l4c_PrepOutput_HDF5_Datasets()



// ---------------------------------------------------------------
// function : l4c_PrepInflate_HDF5_SOC_Datasets()
// purpose  : Adjusts properties of SOC intermediate variables in 
//            anticipation of the INFLATE operation (converting from
//            1D-sparse vector to full 2D global EASEGRID),
//            just prior to writing these intermediate
//            terms back out to the SMAP_L4_C_SOC.h5 intermediate file
// inputs   : void
// returns  : int, status
// revised  : 2014-05-19T14:26:00, jmg, renamed to "l4c_PrepInflate_HDF5_SOC_Datasets()
//            since this logic pertains only to SOC variables.
// notes    : While these properties are hard-coded for now, ideally we
//           want to migrate these to a table driven process,
//           driven of the NSB RIP entries, or other config file etc
int l4c_PrepInflate_HDF5_SOC_Datasets(void)
{
 NsbObj *nsbObj_p = l4c_GetNsbOverseer();
 SesCtl *sesCtl_p = l4c_GetSesCtl();
 char nvpBuf[256] = {0} ; 
 
 PkgMgr *pkgMgr_p = sesCtl_p->l4cPkgMgr_p;
 PkgTheme *pThme_p= NULL;
 PkgObj   *pObj_p = NULL;
 
    
    // -------------------------------------------------------------------------
    // SOC THEME
    pThme_p  = pkgMgr_GetThemeByName(pkgMgr_p,"SMAP_L4_C_SOC");
    
    pObj_p = pkgMgr_GetObjByName(pThme_p,"FT_ROLLING_M03");
    pkgMgr_SetObjGenericPath(pObj_p, sesCtl_p->inputSOCPath );  
    pkgMgr_SetObjDatatype(pObj_p, (int)DFNT_UINT8 );    
    pkgMgr_SetObjResolution(pObj_p, 3 ); // 3 KM
    pkgMgr_SetObjRank(pObj_p, 2 );
    pkgMgr_SetObjNelem(pObj_p, EGv2_GLOBAL_3KM_PIXELS );
    pkgMgr_SetObjFillvalue(pObj_p, 254);

    pObj_p = pkgMgr_GetObjByName(pThme_p,"FT_ROLLING_AGE_M03" );
    pkgMgr_SetObjGenericPath(pObj_p, sesCtl_p->inputSOCPath );  
    pkgMgr_SetObjDatatype(pObj_p, (int)DFNT_UINT8 );    
    pkgMgr_SetObjResolution(pObj_p, 3 );
    pkgMgr_SetObjRank(pObj_p, 2 );
    pkgMgr_SetObjNelem(pObj_p, EGv2_GLOBAL_3KM_PIXELS ); 
    pkgMgr_SetObjFillvalue(pObj_p, 254);
    
    pObj_p = pkgMgr_GetObjByName(pThme_p,nsbGetValueByKey(nsbObj_p,"L4C_INPUT_DATASET_SOC_C0",nvpBuf));
    pkgMgr_SetObjGenericPath(pObj_p, sesCtl_p->inputSOCPath );  
    pkgMgr_SetObjDatatype(pObj_p, (int)DFNT_FLOAT32 );  
    pkgMgr_SetObjResolution(pObj_p, 1 ); // 1 KM
    pkgMgr_SetObjRank(pObj_p, 2 );
    pkgMgr_SetObjNelem(pObj_p, EGv2_GLOBAL_1KM_EW_PIXELS ); 
    pkgMgr_SetObjFillvalue(pObj_p, -9999.0);
    
    pObj_p = pkgMgr_GetObjByName(pThme_p,nsbGetValueByKey(nsbObj_p,"L4C_INPUT_DATASET_SOC_C1",nvpBuf));
    pkgMgr_SetObjGenericPath(pObj_p, sesCtl_p->inputSOCPath );  
    pkgMgr_SetObjDatatype(pObj_p, (int)DFNT_FLOAT32 );  
    pkgMgr_SetObjResolution(pObj_p, 1 );
    pkgMgr_SetObjRank(pObj_p, 2 );
    pkgMgr_SetObjNelem(pObj_p, EGv2_GLOBAL_1KM_EW_PIXELS ); 
    pkgMgr_SetObjFillvalue(pObj_p, -9999.0);

    pObj_p = pkgMgr_GetObjByName(pThme_p,nsbGetValueByKey(nsbObj_p,"L4C_INPUT_DATASET_SOC_C2",nvpBuf)); 
    pkgMgr_SetObjGenericPath(pObj_p, sesCtl_p->inputSOCPath );  
    pkgMgr_SetObjDatatype(pObj_p, (int)DFNT_FLOAT32 );  
    pkgMgr_SetObjResolution(pObj_p, 1 );
    pkgMgr_SetObjRank(pObj_p, 2 );
    pkgMgr_SetObjNelem(pObj_p, EGv2_GLOBAL_1KM_EW_PIXELS ); 
    pkgMgr_SetObjFillvalue(pObj_p, -9999.0);
    
    pObj_p = pkgMgr_GetObjByName(pThme_p,nsbGetValueByKey(nsbObj_p,"L4C_INPUT_DATASET_NPP_SUM",nvpBuf));
    pkgMgr_SetObjGenericPath(pObj_p, sesCtl_p->inputSOCPath );  
    pkgMgr_SetObjDatatype(pObj_p, (int)DFNT_FLOAT32 );  
    pkgMgr_SetObjResolution(pObj_p, 1 );
    pkgMgr_SetObjRank(pObj_p
, 2 );
    pkgMgr_SetObjNelem(pObj_p, EGv2_GLOBAL_1KM_EW_PIXELS ); 
    pkgMgr_SetObjFillvalue(pObj_p, -9999.0);

    l4c_EventCompletionMsg((char *)__func__);

 return 0;
    
} // end:: l4c_PrepInflate_HDF5_SOC_Datasets()
