// ----------------------------------------------------------------
/*
 module       : l4c_DAILY_StringUtilTcf.c (orig was string_util_tcfProc9km.c)
 purpose      : Lucas original string functions for proc 9km
 project      : SMAP L4C science development.
 dependencies : HDF4, FRM_API etc
 revised      : 2016-02-17, updated to use l4c_DAILY_xxx prefix
 revised      : 2014-02-17T,jmg removed fpath_cat, put in array_io.c for now
 revised      : 2014-02-15T14:53:00-07:00,jmg
 author       : joe glassy, Lupine Logic Inc.
 history      : Initial reformating -- laj 09/10/13
 author       : Lucas A. Jones; lucas@ntsg.umt.edu
 notes        : prolog was originally missing, added 2/15/2014
 warning      : <JMG> 2/15/2014, WARNING memory leak here: every string 
               here allocated is never freed due to the way this call interface 
			   separates the malloc from visibility of the 'fpath' outside of 
			   this function. Fix later.

*/

// orig was string_util_tcfProc9km.h"
#include "l4c_DAILY_StringUtilTcf.h"


// ----------------------------------------------------------------
/*Returns a string of nchar from a filename, given a search string and offset from search string base address*/
void string_parse(char *fullstr, char *searchstr, int offset, int nchar, char str[])
{

  char *p;

  /*clear target string*/
  strcpy(str,"\0");

  /*find match then increment base address*/
  p = strstr(fullstr, searchstr);

  if (p != NULL){

    p += offset;

    strcpy(str,p);
    str[nchar] = '\0';

  }

  //returns empty string if string not found

}


// ----------------------------------------------------------------
/*Inserts a string into the middle of a large string at a location
 *determined by the search string.
 */

void insert_str(char *fullstr, char *searchstr, char addstr[])
{

  char  leadStr[MXSTRLEN] = {0};
  char  followStr[MXSTRLEN] = {0};
  char  *p;

  size_t offset;

  //find offset
  offset = strlen(searchstr);

  //find match
  p = strstr(fullstr, searchstr);

  //trim leading string
  strcpy(leadStr,fullstr);
  leadStr[strlen(fullstr)-strlen(p)+offset] = '\0';

  //get following string
  strcpy(followStr,p+offset);

  //paste back together
  strcpy(fullstr,leadStr);
  strcat(fullstr,addstr);
  strcat(fullstr,followStr);

}



// ----------------------------------------------------------------
/*Removes a string from the middle of a larger string at a location
 *determined by the search string.
 */

void remove_str(char *fullstr, char *searchstr, char remstr[])
{

  char  leadStr[MXSTRLEN] = {0};
  char  followStr[MXSTRLEN] = {0};
  char  *p;

  size_t offset;

  //find offset
  offset = strlen(searchstr);

  //find match
  p = strstr(fullstr, searchstr);

  //trim leading string
  strcpy(leadStr,fullstr);
  leadStr[strlen(fullstr)-strlen(p)+offset] = '\0';

  //get following string
  strcpy(followStr,p+offset);

  //get offset to ommit sub-string
  offset = strlen(remstr);

  //paste back together
  strcpy(fullstr,leadStr);
  strcat(fullstr,followStr+offset);

}


// ----------------------------------------------------------------
/*Removes the end of a larger string at a location
 *determined by the search string.
 */

void string_chop(char *fullstr, char *searchstr, int offset)
{

  char  leadStr[MXSTRLEN] = {0};
  char  *p;

  //find match
  p = strstr(fullstr, searchstr);

  //trim leading string
  strcpy(leadStr,fullstr);
  leadStr[strlen(fullstr)-strlen(p)+offset] = '\0';

  strcpy(fullstr,leadStr);

}



// ----------------------------------------------------------------
/*Retrieve data-type from filename extension*/

int32 dtype_from_dtypestr(const char dtype_str[])
{

  int32   dataType;

  if (strcmp("int8",dtype_str)==0)    dataType = DFNT_INT8;
  else if (strcmp("uint8",dtype_str)==0)    dataType = DFNT_UINT8;
  else if (strcmp("int16",dtype_str)==0)    dataType = DFNT_INT16;
  else if (strcmp("uint16",dtype_str)==0)    dataType = DFNT_UINT16;
  else if (strcmp("int32",dtype_str)==0)    dataType = DFNT_INT32;
  else if (strcmp("uint32",dtype_str)==0)    dataType = DFNT_UINT32;
  else if (strcmp("flt32",dtype_str)==0)    dataType = DFNT_FLOAT32;
  else if (strcmp("flt64",dtype_str)==0)    dataType = DFNT_FLOAT64;
  else{
    fprintf(stderr, "ERROR: undefined data type: %s\n", dtype_str);
    dataType = DFNT_NONE;
  }

  return(dataType);
}


// ----------------------------------------------------------------
/*Searches a filename path string for its final '/' token and returns
 *the file name (whatever is right of the final '/' token)
 *and also returns the data type (whatever is right of the final '.' token
 */

void parse_fpath(char *fullPath, char *fileName, char *dtype)
{

  char copyPath[MXSTRLEN] = {0};

  char delim_path[] = "/";
  char delim_ext[] = ".";
  char *p;


  /*extract file name from path
   *using the "/" token.
   */

  strcpy(copyPath, fullPath);
  strcat(copyPath,"/"); //need a trailing token for method to work

  p = strtok(copyPath, delim_path);

  while ((p = strtok(NULL,delim_path)) != NULL)
    {
      strcpy(fileName,p);
    }

  //  printf("\n filename: [%s]\n", fileName);

  /*extract data type extension from file name
   *using the "." token.
   */
  strcpy(dtype, fileName);
  p = strtok(dtype, delim_ext);

  strcpy(fileName, p); //get fileName without extension

  while ((p = strtok(NULL,delim_ext)) != NULL)
    {
      strcpy(dtype,p);
    }

  //  printf("\n datatype: [%s]\n", dtype);

}


// ----------------------------------------------------------------
/*Strips off trailing post-fixes following a given token
 */
void strip_fname(char *fileName)
{

  char copyPath[MXSTRLEN] = {0};

  char delim[] = "_";

  char *p;


  strcpy(copyPath, fileName);

  p = strtok(copyPath, delim);
  strcpy(fileName,p);
  strcat(fileName,delim);

  while ((p = strtok(NULL,delim)) != NULL)
    {
      strcat(fileName,p);
      strcat(fileName,delim);
    }

  //necessary to ovewrite the trailing token
  fileName[strlen(fileName)-1] = '\0';

  //  printf("\n filename: [%s]\n", fileName);

} // end:: strip_fname()


