// --------------------------------------------------------------
// function: l4c_DAILY_bplut.h
// purpose : Implements function prototypes for l4c_MODIS_bplut.c
// author  : joe glassy
// revised : 2017-06-05, changed call interface to l4c_BPLUT_Read() to include
//           sesCtl_p.
// revised : v5.0.1-2014-08-06T17:00:00-07:00, <LAJ>, Add FT-related BPLUT columns.
// revised : 4.2.7-2014-02-22T12:26, split out scan and allocation of BPLUT
//           to better facilitate multiple uses (or READS) of BPLUT within
//           a complex application, without the risk of redundant allocations
//
// revised : 4.2.6-2014-02-19T23:44:00-07:00 (RunParamAdt added, and fix-ups)
// history : adapted from tcf_spin v3.2
// notes   : Version enforced on fixed UUID 6640b8c0-9a01-11e3-a29d-000c29ddf1f0
// notes   : This module implements the text I/O method on SMAP_BPLUT_2014-02-11.csv
//           Later in production the BPLUT will be re-deployed in the SMAP_L4_C_Ancillary.h5

// notes   : TBD: In future, move required macros out of tcf_const.h into here and
//           move definition of main BPLUT structure into here to keep ALL
//           BPLUT components together!
// notes   : The BPLUT ADT itself is defined in tcf_struct.h, but is planned
//           to be moved into this l4c_xxx_bplut.h header


#ifndef L4C_BPLUT_DEF
#define L4C_BPLUT_DEF 1

#include "l4c_DAILY.h"
#include "l4c_DAILY_prot.h"

// <JMG> added 2014-02-22, to facilitate multiple uses in one app
int   l4c_BPLUT_Scan(FILE *bplutf);
// separated out the allocate step to avoid this occuring multiple times

// Allocates a module-level-static bpLUT_Master_p so does NOT return it
// since it is already visible to all functions in the module file
int l4c_BPLUT_Allocate(const int nValidRecs,const char *bpLUT_Label_p);

// <JMG> 2017-06-05 changed call interface to include sesCtl_p to facilitate
//  later runtime hard-verication of BPLUT version via internal
//  NVP L4C_BPLUT_UUID from RIP matched to UUID inside CSV file itself.

//  Perform BPLUT read only, now separated from pre-scan to count, and 
//  separated out from the allocation...
BPLUT *l4c_BPLUT_Read(SesCtl *sesCtl_p,int *nBplutRecords);

BPLUT *l4c_GetBPLUT_Private(void);
BPLUT *l4c_GetBPLUT_Record(const char *landcoverLutKey_p,const int nValidBPLUT_Records);
BPLUT *l4c_GetBPLUT_ByIndex(const int landcoverIntegerKey,const int nValidBPLUT_Records);
void l4c_UnitTest_BPLUT(const int nRec);

int l4c_BPLUT_Report(FILE *outf,const int nPftClasses);


#endif
