// --------------------------------------------------------------------
// module : l4c_DAILY_grid_const.h (was l4c_gridconst.h
// purpose: define all L4_C grid constants 
// revised: 2016-01-04T14:04, added doc only
// revised: 2014-02-20T12:18:00, renamed to l4c_DAILY_grid_const.h to match others
// revised: 2014-02-19T23:57:00, added 3km grid constants
// history: 2013-05-19T10:44:00-07:00
// author : joe glassy, Lupine Logic Inc.
// notes  : n/a


#ifndef L4C_GridConstDef 
#define L4C_GridConstDef 1

// GRID CONSTANTS

// Define EASE-Grid 2.0 "EGv2" spatial extents constants 

// Total number of cells in EGv2 1km land domain (LDOM9KM*NCELL2)
#define EGv2_LDOM1KM   134787240  
// Total number of cells in EGv2 3km land domain 
#define EGv2_LDOM3KM    14976360
// Total number of cells in EGv2 9km land domain
#define EGv2_LDOM9KM     1664040

// EASE-Grid v2 1km CELL constants 
#define EGv2_GLOBAL_1KM_NS_PIXELS      14616
#define EGv2_GLOBAL_1KM_EW_PIXELS      34704   
#define EGv2_GLOBAL_1KM_PIXELS     507233664 

// EASE-Grid v2 3km CELL constants
#define EGv2_GLOBAL_3KM_NS_PIXELS       4872
#define EGv2_GLOBAL_3KM_EW_PIXELS      11568
#define EGv2_GLOBAL_3KM_PIXELS      56359296

// EASE-Grid v2 9km CELL constants
#define EGv2_GLOBAL_9KM_NS_PIXELS       1624
#define EGv2_GLOBAL_9KM_EW_PIXELS       3856
#define EGv2_GLOBAL_9KM_PIXELS       6262144

// MODIS sinusoidal CELL or PIXEL related constants 
// Define MODIS (collection 5 and later) sinusoidal CELL or PIXEL constants
#define MODIS_GLOBAL_1KM_NS_PIXELS        21600
#define MODIS_GLOBAL_1KM_EW_PIXELS        43200
#define MODIS_GLOBAL_1KM_SUM_PIXELS   933120000

// define MODIS 10-degree TILE pixel parameters
#define MODIS_10Deg_1KM_NS_PIXELS         1200 
#define MODIS_10Deg_1KM_EW_PIXELS         1200 
#define MODIS_1KM_SUM_PIXELS           1440000

// Misc other grid and coordinate transformation constants of interest. Not used yet.
#define MODIS_VV_TILES                      18
#define MODIS_HH_TILES                      36

#define MODIS_1KM_GLOBAL_ROWS            21600
#define MODIS_1KM_GLOBAL_COLS            43200
#define MODIS_1KM_GLOBAL_PIXELS      933120000

// MERRA (no longer needed)
#define MERRA_ROWS        361
#define MERRA_COLS        540

// GEOS5 Forward Processing (FP) is currently at (verify:) geographic
// 1/4 degree resolution
// 1-deg is {-90..+90}=180 along lat axes, {-180...+180}=360 along longitudinal axes
// This is expected to change when Rolf produces a finer res produce
#define FP_GEOS5_ROWS     720
#define FP_GEOS5_COLS    1400

#endif
