// --------------------------------------------------------------
// function: l4c_DAILY_prot.h
// purpose : declare function prototypes, by module or package
// author  : joe glassy, Lupine Logic Inc.
// revised : 2020-07-4T10:35, changed call signature of l4c_Hdf5_AssignFillValue() to
//           accept pObj_p directly, since its already available upstream
// revised : 2016-03-04T11:52, added to condition uint8 qa_count arrays for use by
//           l4c_PostProc9km_WriteObsMinMaxMean(
// revised : 2016-03-02, added new RMSE specific diagnostic monitor        
//                int l4c_PostProc9km_DiagMonitor() to check for neg RMSE values
// revised : 2016-02-17, updated more modules to use l4c_DAILY_xxx prefix.
// revised : 2016-02-11, in l4c_DAILY_hdf5.c module, added l4c_Hdf5_AssignFillValue()
//           to address correct matching a _FillValue datatype with each datasets datatype.
// revised : 2015-10-15, changed name of l4c_ReportInvalidNee(void)
//           to l4c_GetInvalidEventCounts()
// revised : 2014-11-18, encapsulate JUST paths to SesPathAdt, to split 
//           up too-broad SesCtl into more narrowly focused ADTs.
// revised : 2014-11-03, added l4c_SesInitPrePostReusableImages() and
//           l4c_SesReleasePrePostReusableImages() to support multi-day mode
// history : 2014-10-06, fixed name of l4c_Hdf5_AddFormalMetadata() and
//           added attribute writes for missing ISO elements Joe A reported
//           "CompletenessOmission" and "DomainConsistency"
// history : 2014-09-30, refactored SOC update logic, split out into sep fx
// history : 2014-09-12, Merra climatology renamed to generic LongtermClimatology
// history : 2014-09-03T12:32, deprecated entire ".qa" file, consolidated
//           all QA data into a QA group now inside "_mdl_".h5 file.
// history : 2014-08-26T10:31, added l4c_GetExecutableVersionId(void)
//           to standardize across all components.
// history : 2014-08-15T11: 15:00, added l4c_writeNvpAsAttribute() as
//           borrowed from l4c_MODIS code set.
// history : 2014-08-13T16:22:00 added l4c_Ses_DatestampLog to enable
//           datetime stampling log file, and 
//           added l4c_Hdf5_Write_CF_Attrib()
// history : <JMG> 2014-08-08T11:34, added l4c_SesVerifyExistance()
// history : 2014-08-07T13:55:00, <JMG> added int l4c_Update_SoilrootzoneMinMax()
//           and replaced surflag_3km[] with ft_state_um_3km[]
//           and reviewed all <LAJ> changes to implement FT term in gpp calculation,
//           and introduced new BPLUT dated 8/7/2014
// history : 2014-07-23T120:00:00, <LAJ> modified do aggregation of 1km to 9km within forward step
//         : This required addition of several new function definitions in the PostProc9km module, and 
//         : removal or modification of prevoius functions in the module.
// history : 2014-07-21T1:40, <JMG> added l4c_AssignBits(). Also available
//           in UM Framework as frm_utilAssignBits().
// history : 2014-06-24T13:16, added l4c_DAILY_WriteMetadata.c prototypes.
// history : 2014-04-10T15:00, added l4c_SetBits(), l4c_GetBits() from MODIS MUM API.
// history : 2014-03-07T20:52:00, history deflate 1km,3km,9km functions to return
// history : 2014-03-02T16:40, history deflate 1km,3km,9km functions to return
//           a check count of pixels processed
// history : 2014-02-27T12:21:00, re-org PkgInit, split out into sep. functions
// history : 2014-02-26T117:35:00:00, added hard coded assignments for DEFLATE init
// history : 4.2.6-2014-02-19T23:44:00-07:00 (RunParamAdt added, and fix-ups)
// history : 4.1.6-2014-02-12T18:14:00, added calc_vpd in l4c_DAILY_sci_util.c
// history : 4.1.5-2014-01-27T12:57:00
// history : 4.1.4-2014-01-14T17:25:00-07:00,wheat,jmg
// history : 2013-11-20T17:04:00-07:00
// history : 2013-11-01T09:31:00-07:00
// notes   : n/a

#ifndef L4C_DAILY_Prot_DEF 
#define L4C_DAILY_Prot_DEF 1

#include "l4c_DAILY.h"

// -- IMPORT common framework components
//    note: these have now been integrated into their own independent
//       header group, encapsulated by frm_api.h

// IMPORT NSB prototypes, a component of the UM Framework 
#include "nsb_api_prot.h"

// IMPORT PkgMgr prototypes, a component of the UM Framework 
#include "pkg_mgr_prot.h"

// IMPORT symbols needed for PostProc9km
#include "l4c_DAILY_PostProcessor.h"

// --module: l4c_Daily_ses.c ---------------------------------------

// <JMG> added 2014-08-26T10:31
char *l4c_GetExecutableVersionId(void);

AppControlAdt *l4c_GetLegacyCONT(void);
// (accessors) full 2D grid version of primary 
//   Dataset Packages namespace pattern: XXXAdt_2D
PftAdt            *l4c_GetPftAdt_2D(void);
MetArrayAdt       *l4c_GetMetArrayAdt_2D(void);
EcoStateArrayAdt  *l4c_GetEcoStateArrayAdt_2D(void);
AncImagePkgAdt    *l4c_GetAncImagePkgAdt_2D(void);
// <JMG> 2014-07-23, added 'get' accessor, for RunParamAdt access
RunParamAdt *l4c_GetRunParam(void);
SesCtl  *l4c_GetSesCtl(void);
NsbObj  *l4c_GetNsbOverseer(void);
PkgMgr  *l4c_GetPkgMgrOverseer(void);

// <JMG> 2014-11-18, encapsulate JUST paths, to begin splitting up too-broad SesCtl
SesPkgPathAdt   *l4c_GetPathAdtOverseer(void) ;
SesPkgHdf5IdAdt *l4c_GetHdf5IdOverseer(void);

// New authentication enforced "gateway" to filter who gets
// access to the "keys to the kingdom"
SesCtl  *l4c_GetSesControl(const int requesterCategory, const int requesterId);

// <JMG> 2014-10-06T16:28, added access fx to container for 
// ISO metadata elements such as DomainConsistency,CompletenessOmission
// etc.
L4C_ISO_MetaElements *l4c_GetISO_MetaElements(void);


// <JMG> 2014-07-23, added get accessor for QcStateLimitAdt master instance
QcStateLimitAdt *l4c_GetQcStateLimits(void) ;


char   *l4c_SesQualifyPathWithRoot(const char *rootDirPath_p,
                char *origPath_p);

// returns cumulative seconds elapsed since session began
double  l4c_EventCompletionMsg(const char *thisFunction_p);
// posts high-water-mark message then performs early, orderly shutdown
double  l4c_EventHighWaterMarkMsg(const char *thisFunction_p);

// <JMG> added 3/14/2014 to datetime stamp final log file 
char    *l4c_Ses_LogLabelTime(const time_t finalSec,char *finalLogPath);
// <JMG> added 08/12/2014 to enable datetime stampling log file
int     l4c_Ses_DatestampLog(SesCtl *sesCtl_p);

int     l4c_Ses_PkgMgr_Init(void);
time_t  l4c_InitSession(int argc,char *argv[]);
void    l4c_CloseSession(void);

// <JMG> 2014-11-03, added, to support multi-day mode 
int l4c_SesInitPrePostReusableImages(void);
int l4c_SesReleasePrePostReusableImages(void);

// --module: l4c_DAILY_rip.c   -----------------------------------------
// NVP (name-value-pair) from RIP, related processing
// <JMG> 2017-06-05, changed return type of 'collect' RIP parsing 
// h andlers to NSB
NsbObj  *l4c_SesCollectCoreNvps(SesCtl *sesCtl_p);
NsbObj  *l4c_SesCollectNVPs(void);
int     l4c_SesCollectOutputNVPs(void);

void    l4c_SesVerifyNVPs(NsbObj *nsb_p);

// <JMG> 2015-10-22T12:42, added dayOfYear as co-key for LUT search
int l4c_SetFparClimatologyDatasetName(const int dayOfyear,
    const int compositePeriodIdx,char *lutPath_p,char *dsetName_p);

 // <JMG> 7/23/2014, added prototypes, testing is TBD
int     l4c_ParseRipDatasetProperties(const char *dataPropTheme,const char *dataPropStr);

// <JMG> 2014-08-08T11:34, added new
int l4c_SesVerifyExistance(void) ;


// --module: l4c_DAILY_DatasetProperties.c   ---------------------------
// <JMG> added 09/10/2014..
int l4c_AssignOutputDatasetProperties(void);


// --module: l4c_DAILY_Action.c   --------------------------------------
AncImagePkgAdt   *l4c_GetAncImagePkgAdt(void);

// <JMG> 2014-11-03, added to better organize pre and post phase image
// in prep for multiple-day processing mode.
PrePostImgPkgAdt *l4c_GetPrePostImagePkgAdt(void) ;

// EcoStateAdt package of scalars
EcoStateAdt *l4c_GetEcoStateAdt(void);
// EcoStateArrayAdt package of 1D-sparse vectors
EcoStateArrayAdt *l4c_GetEcoStateArrayAdt(void);
// MetArrayAdt package of 1D-sparse vectors
MetArrayAdt      *l4c_GetMetArrayAdt(void);
// MultCoefAdt package..
MultCoefAdt *l4c_GetMultCoefAdt(void);
// PftAdt package...
PftAdt   *l4c_GetPftAdt(void);

// this replaces tcfAction_forwardStep with HDF5 enabled I/O and 
// reorganized memory allocs and frees
void  l4c_Action_forwardStep(void);
int   l4c_AncPkgAlloc(void);

// --module: l4c_DAILY_PrepOutput.c ------------------------------------

int l4c_PrepOutput_HDF5_Datasets(void);
// <JMG> 2014-05-19, renamed to 'SOC' as it pertains only to SOC vars
int l4c_PrepInflate_HDF5_SOC_Datasets(void);

// --module: l4c_DAILY_PkgMgrInit.c -------------------------------
int l4c_PkgMgr_Init(const int nThemes);

// <JMG> 2014-06-24, added
void l4c_PkgInit_Metagroups(PkgMgr *pkgMgr_p);

// <JMG> 2014-05-19, all functions originally in l4c_DAILY_PkgMgrInit_Themes.c
// have now been consolidated into l4c_DAILY_PkgMgrInit.c
void l4c_PkgInit_SMAP_INPUT_HDF5_FILES(PkgTheme *pThme_p,char *themeName_p);

// void l4c_PkgInit_AppControlAdt(PkgTheme *pkgTheme_p,char *themeName_p);
// void l4c_PkgInit_PftAdt(PkgTheme *pkgTheme_p,char *themeName_p);
// void l4c_PkgInit_MetAdt(PkgTheme *pkgTheme_p,char *themeName_p);
void l4c_PkgInit_MetArrayAdt(PkgTheme *pkgTheme_p,char *themeName_p);
// void l4c_PkgInit_EcoStateAdt(PkgTheme *pkgTheme_p,char *themeName_p);
void l4c_PkgInit_EcoStateArrayAdt(PkgTheme *pkgTheme_p,char *themeName_p);
// void l4c_PkgInit_EcoStateQaAdt(PkgTheme *pkgTheme_p,char *themeName_p);
// void l4c_PkgInit_MultCoefAdt(PkgTheme *pkgTheme_p,char *themeName_p);
// void l4c_PkgInit_ParamAdt(PkgTheme *pkgTheme_p,char *themeName_p);
void l4c_PkgInit_L4C_OUTPUTS(PkgTheme *pkgTheme_p,char *themeName_p);
void l4c_PkgInit_SMAP_L4_C_Ancillary(PkgTheme *pkgTheme_p,char *themeName_p);
void l4c_PkgInit_SMAP_L4_C_SOC(PkgTheme *pkgTheme_p,char *themeName_p        );
void l4c_PkgInit_SMAP_L4_C_MET(PkgTheme *pkgTheme_p,char *themeName_p        );

void l4c_PkgInit_SMAP_L4_C_FPAR_8DAY(PkgTheme *pkgTheme_p,char *themeName_p);

void l4c_PkgInit_SMAP_FparClimatology(PkgTheme *pkgTheme_p,char *themeName_p );
// <JMG> 2014-09-12 renamed to generic "LongtermClimatology"
void l4c_PkgInit_SMAP_LongtermClimatology(PkgTheme *pkgTheme_p,char *themeName_p);
void l4c_PkgInit_PRE_PROC_DEFLATE_1KM(PkgTheme *pkgTheme_p,char *themeName_p );
void l4c_PkgInit_POST_PROC_INFLATE_9KM(PkgTheme *pkgTheme_p,char *themeName_p);

// <JMG> 2014-05-19, now sets file level properties into each respective
// theme named after the file, e.g. "SMAP_L4_C_{file_qualifier}" for 
// self-referential access
int l4c_PkgInit_SetFileProperties(void);


// --module: l4c_DAILY.c -----------------------------------
// <JMG> 2014-11-03, added placeholder logic for future multi-day mode


// --module: l4c_DAILY_ForwardStep.c------------------------------------
void l4c_ModDriverGrid_forwardStep(int doy, PftAdt *pftObj_p,
    MetArrayAdt *metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);

// --module: l4c_DAILY_DailyDriver.c -----------------------------------
// <JMG> 2014-09-04, renamed...l4c_GridManager
// used for official GMAO daily production

int l4c_DailyDriver(const int runDay, int appProcessingPhase) ;

// --module: l4c_GridPre_Deflate.c -------------------------------------

// L4_C variants of DEFLATE logic
size_t l4c_deflate_1km(uint16 *row_1D_LUT_p,uint16 *col_1D_LUT_p, void *src_p,void *dest_p,const int hdf4Datatype);
size_t l4c_deflate_3km(uint16 *row_1D_LUT_p,uint16 *col_1D_LUT_p, void *src_p,void *dest_p,const int hdf4Datatype);
size_t l4c_deflate_9km(uint16 *row_1D_LUT_p,uint16 *col_1D_LUT_p, void *src_p,void *dest_p,const int hdf4Datatype);

// --module: l4c_PreProc1km_func.c -------------------------------------

// <JMG> 2015-09-06T18:36:00 added, as diagnostic for f/t
int l4c_SummarizeFreezeThaw(SesCtl *sesCtl_p, uint8 *deflated_1DSparseVec_p) ;

// DEPRECATED: int l4c_PrepDeflate_HDF5_Datasets(PkgMgr *pkgMgr_p, char **dsetNameTable_p,const int nPreProcVariables);
// new 5/16/2016, replaces above
int l4c_PrepDeflate_DatasetDriver(PkgMgr *pkgMgr_p, NsbObj *nsbObj_p);

// perform initial read of full 2D grid, as input to DEFLATE processes
int l4c_Read_SOC_2D(EcoStateArrayAdt *ecoStateArrayObj_2D_p,char *socHdf5Path);
int l4c_Read_FPAR_8DAY_2D(MetArrayAdt *metArrayObj_2D_p,char *fparHdf5Path);
int l4c_Read_Met_2D(MetArrayAdt *metArrayObj_2D_p,const char *metPath);

size_t  l4c_PreProc_Calculate_VPD(MetArrayAdt *metArrayObj_p);

int   l4c_ReadAnc_M09land_row_col_9km(void);

int l4c_GetThemePropertySummary(const char *hdf5_datasetName_p,
    int *childDatatype, int *childResolution, size_t *child_nElements,size_t *objSiz);
    

void *l4c_PreProcAllocate_1DSparseVector(const char *h5datasetName_p);

int l4c_PreProc_Setup(PkgMgr *pkgMgr_p, const int nPreProcVariables);

char **l4c_PreProc_MakeHdf5ThemeTable(const int nPreProcVariables);

void l4c_PreProc_Read_2D_Image(const hid_t curr_file_id,char *datasetName_p,char *callerLbl_p);

int  l4c_PreProc_Bind_1D_DeflatedVars(char *datasetName_p,
                            void *deflated_1DSparseVec_p);

// <JMG> 2014-05-19, deprecated use of 2 string tables, now get dataset
// names from PkgMgr...
int l4c_PreProc_ByLayerDriver(const int nTotalToDeflate) ;

// this version will only work on very large memory platforms
int l4c_PreProc_LargeMemPlatform(EcoStateArrayAdt *ecoStateArrayObj_2D_p,
                    MetArrayAdt *metArrayObj_2D_p);

// --module: l4c_GridPost_Inflate.c ------------------------------------
void l4c_AssignFillvalue(const int hdf4Datatype, const size_t atSlot,void *vDest_p); 
void l4c_inflate_init_9km(void *dest_p, const int hdf4Datatype);
void l4c_inflate_9km(void *src_p, void *dest_p, const int hdf4Datatype);
void l4c_inflate_init_3km(void *dest_p, const int hdf4Datatype);
void l4c_inflate_3km(void *src_p, void *dest_p,const int hdf4Datatype);
void l4c_inflate_init_1km(void *dest_p, const int hdf4Datatype) ;
void l4c_inflate_1km(void *src_p, void *dest_p,const int hdf4Datatype);

// --module: l4c_DAILY_BindToPkgMgr.c -----------------------------

// function performing the array variable level binding
int l4c_BindPkg_AncillaryStatic(AncImagePkgAdt *ancImagePkgObj_p,
            MetArrayAdt *metArrayObj_p);

// <LAJ> Should be able to accept 'uint8' data type in addition to 'float32' 
int  l4c_BindPkg_EcoStateArrays(EcoStateArrayAdt *ecoStateArrayObj_p);
int  l4c_BindPkg_SOC_and_Npp(EcoStateArrayAdt *ecoStateArrayObj_p);
int  l4c_BindPkg_SOC_Intermediate(AncImagePkgAdt *ancImagePkgObj_p,
            MetArrayAdt *metArrayObj_p) ;

int  l4c_BindPkg_MetArrays( MetArrayAdt *metArrayObj_p);


// --module: l4c_DAILY_tcfMem.c ----------------------------------------
void tcfMem_stateArrayFree(EcoStateArrayAdt *ecoStateArrayObj_p);
void tcfMem_metArrayFree(MetArrayAdt *metArrayObj_p);
void tcfMem_stateArrayAlloc(uint32 nelem1km);
// MetArrayAdt tcfMem_metArrayAlloc(uint32 nelem9km);
void tcfMem_metArrayAlloc(uint32 nelem9km);

// --module: l4c_DAILY_hdf5.c ------------------------------------------

// <JMG> 2014-09-03T16:31, added to improve sync of all HDF5
// output dataset attribute definitions
int l4c_AssignOutputDatasetProperties(void);

// <JMG> 2016-02-11T10:43, added l4c_Hdf5_AssignFillValue()
// <JMG> 2020-07-14T10:36 changed call to use pObj_p since its already available
int l4c_Hdf5_AssignFillValue(hid_t dset_id,PkgObj *pObj_p);

Hdf5GridPropAdt *l4c_GetHdf5GridProp(void);

// <JMG> 2014-09-03T16:25:00, added
hid_t l4c_TIVDatatypeToH5TDatatype(const int tivDatatype);
int   l4c_H5TDatatypeToTIVDatatype(const hid_t h5tDatatype);

int l4c_DatatypeStringTo_Hdf4TypeId(const char *dtype_str);

char *l4c_TIVDatatypeToDatatypeString(const int TIV_datatypeId,char *typBuf);

hid_t l4c_Hdf5_CreateGroup(const hid_t owner_file_id,char *groupLabel_p);

int l4c_Hdf5_CreateAllOutputGroups(const hid_t owner_file_id, const int doCreateMetataGroups, hid_t *groupIdAry_p);

hid_t l4c_Hdf5_CreateOutput(void);          

// CREATE initially empty dataset definitions
int l4c_Hdf5_CreateDatasets(hid_t file_id);

// new list-wise method for creating all datasets and adding their dataset metadata
int l4c_Hdf5_CreateDatasetsListwise(hid_t file_id);

// <JMG> 2014-08-13 added l4c_Hdf5_Write_CF_Attrib()
void l4c_Hdf5_Write_CF_Attrib(hid_t loc_id,char *themeName_p,char *datasetName_p);

// deprecated: l4c_Hdf5_ReadDataset();

int l4c_Hdf5_WriteData(const char *callerLbl,const hid_t thisFile_id,const int rank,const hsize_t *dims,
        const hid_t dsTypeId,char *datasetName_p,void *dataBuffer_p);

int l4c_Hdf5_CreateAndWriteDataset(const char *callerLbl_p,const hid_t file_id,const char *dsName_p,
        const hid_t dsType_id,const hid_t dspace_id,hid_t dcpl,void *dsData_p);

int l4c_Hdf5_WriteDataset(const char *callerLbl_p,const hid_t file_id,const char *dsName_p,
        const hid_t dataset_id, const hid_t dsType_id,void *dsData_p);

hid_t l4c_Hdf5_CreateAttribute(const hid_t owner_id,int tivDatatypeIdentifier,
        const char *objName_p,const char *attribName_p, void *attribData_p);

void l4c_Hdf5_CreateAttributeString(hid_t owner_id,const char *attribName_p,
        char *attribStringData_p);

void l4c_Hdf5_AddFileMetadata(void);

// <JMG> 2014-10-30, split out from AddFileMetadata
void l4c_Populate_ISO_MetadataElements(void);
int  l4c_Hdf5_Add_ISO_FileMetadata(const hid_t file_id);

int l4c_Hdf5_CloseSession(Hdf5GridPropAdt *hGridPropObj_p,
        const hid_t l4cMainOutput_file_id);

int l4c_Hdf5_WriteLatLon_CoordVars(const hid_t file_id);

// <JMG> added 2014-08-15T11:14, borrowed from l4c_MODIS
hid_t l4c_writeNvpAsAttribute(hid_t file_id,const char *keyword,const char *attrForceName);

// --module: l4c_DAILY_WriteMetadata.c -------------------------

// <JMG> 2014-10-06, moved to l4c_DAILY_WriteMetadata.c for consistency
void l4c_Hdf5_AddFileMetadata(void);
// corrected name to _Hdf5_
int  l4c_Hdf5_AddFormalMetadata(const hid_t file_id);

// <JMG> 2016-03-03 added to begin consolidating writes of all 
//  dataset attributes to occur in one module file, l4c_DAILY_WriteMetadata
int  l4c_MetadataWriteDataset(hid_t file_id) ;

void l4c_MetadataWriteGroup_CRID(const hid_t file_id);
void l4c_MetadataWriteGroup_SeriesIdentification(const hid_t file_id);
void l4c_MetadataWriteGroup_Config(const hid_t file_id);
void l4c_MetadataWriteGroup_DatasetIdentification(const hid_t file_id);
void l4c_MetadataWriteGroup_Extent(const hid_t file_id);
void l4c_MetadataWriteGroup_GridSpatialRepresentation(const hid_t file_id);
void l4c_MetadataWriteGroup_ProcessStep(const hid_t file_id);

// --module: l4c_DAILY_Hdf5_Read_and_Updates.c -----------------------
// <JMG> 2014-11-11, changed call interface, key on calendarLabel only now
int l4c_SOC_or_FT_CalendarUpdate(const int opMode,  const char *calendarLabel_p,const int curr_doy);


// <JMG> 2014-09-30, single pool at a time processor, split out from monolithic
int l4c_SOC_UpdatePoolDriver(char *socPoolLabel_p,EcoStateArrayAdt *ecoStateArrayObj_p,float32 *post2D_M01_p);
int l4c_FT_RollingDriver(EcoStateArrayAdt *ecoStateArrayObj_p,char *ftDatasetName_p,uint8 *imageBuffer2D_M03_p);

// These are the HDF5 equivalent of saveCstate(), saveFTRolling state
int  l4c_SOC_Update_Intermediates(EcoStateArrayAdt *ecoStateArrayObj_p);

// <JMG> 2014-08-07, added to support SMAP NatureRun soil rootzone min,max updates
size_t l4c_SoilrootzoneMinMaxUpdate(MetArrayAdt *metArrayObj_p,size_t *nMinChanged,size_t *nMaxChanged);
int    l4c_SoilrootzoneMinMaxDriver(MetArrayAdt *metArrayObj_p) ;

int  l4c_HDF5_ReadStatic_SMRZ_MinMax(MetArrayAdt *metArrayObj_p);
int l4c_HDF5_Read_SOC_and_FT(EcoStateArrayAdt *ecoStateArrayObj_p);
int l4c_HDF5_Read_Met(MetArrayAdt *metArrayObj_p);

// --module: l4c_DAILY_UpdateIntermediates.c -----------------------

// tcfMem_saveCstate(), left in for backward compatibility
void l4c_saveCstate(char pname[], char doyStr[], uint32 nelem, EcoStateArrayAdt *ecoStateArrayObj_p);

// replacement for tcfMem_saveFtRollingState()
void l4c_saveFtRollingState(char doyStr[], uint32 nelem, EcoStateArrayAdt *ecoStateArrayObj_p);

// replacement, for updating SOC and FT rolling states
int l4c_UpdateIntermediates(char *socFlag,char *doyStr,
        int nelem1kmPerTask,int nelem3kmPerTask, EcoStateArrayAdt *ecoStateArrayObj_p);


// --module: l4c_DAILY_LoadInputs.c -------------------------------
// this is the legacy code, left in for backward compatibility
int l4c_LoadInputs(MetArrayAdt *metArrayObj_p,EcoStateArrayAdt *ecoStateArrayObj_p);

// --module: l4c_EASEGRID_Transforms.c -----------------------------

int easegrid2_wgs84_fwd(const int mapProjectionCode,const float64 lat,
            const float64 lon, int *ithRow, int *ithColumn);

int easegrid2_wgs84_inv(const int32 mapProjectionCode,
            const int ithRow,const int ithColumn,float64 *lat,float64 *lon);


// --module: l4c_DAILY_sci.c ---------------------------------------

int     l4c_TestBooleanStr(const char *optionSwitch_p);

int     l4c_GetISO8601_datetime(time_t usersSec,char *loc_dtStr_p,char *utc_dtStr_p);

int     l4c_are_floats_equal(const float a,const  float b);
int     l4c_are_doubles_equal(const double a,const double b);
size_t  l4c_sizOfDatatype(const int hdf4Datatype);

float32 l4c_calc_vpd(const float32 tavg,const float32 qv2m,const float32 ps);


// added: 4/10/2014 <JMG>
uint8  l4c_testByteorder(void) ;
// use to retrieve 1 or more bits from a word
uint32 l4c_GetBits(const int16 startPos,const int16 endPos, void *data,size_t dataSize);
// added: 7/21/2014, <JMG>. Use to set larger integer quantities, like PFT 1..8, into a word
uint32 l4c_AssignBits(const int16 startPos,const int16 endPos, void *destWord, size_t destWordSize,
                      void *newValueMask, size_t newValueMaskSize);
// best to use this to set individual (single) bits in a word
uint32 l4c_SetBits(const int16 startPos,const int16 endPos, void *data,size_t dataSize);


// --module: l4c_DAILY_Diagnostics.c -----------------------------------
float32 l4c_SummarizePixels(PostProcMetrics *postMetrics_p,
        const char *dsetName_p, const hid_t hdf5Id,
        const size_t pixelCount,const float32 fillValue,float32 *sciAry_p);
        
// prototype quick-and-dirty diagnostic image generation, uses: genDiagImg.py
int l4c_DiagSparseArrayToImage(char *imgLabel_p,char *outPath_p,
    const float32 missingValue,const size_t resKmPixels,
    int const tivDatatype,void *sciArray_p);        

// <JMG> 2016-01-07 added...
int l4c_SesCtl_Display(SesCtl *sesCtl_p,FILE *diagf) ;

// TBD: merge &* migrate the rest of ALL symbols in tcf_func.h into main L4_C headers...
// orig was tcf_func.h
#include "l4c_DAILY_tcf_prot.h"

// <LAJ> 8/6/2014, This function would have be with the other multiplier functions in "tcf_func.h"
//                 but since it's new as of today and I see a TBD note to merge - I'll place it's prototype here.  
float64 *tcfModMult_gppFt(BPLUT *bp_p, uint8 ft);

// --module: l4c_DAILY_ModFunc.c  -----------------------------------------
int  tcfModFunc_isInCell(uint8 pft_index, MetAdt metObj, EcoStateAdt ecoStateObj);
int tcfModFunc_switchFTmethod(uint32 s3km, MetAdt *metObj_p, MetArrayAdt *metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);
int  tcfModFunc_switchFparVi(BPLUT *bp_p, uint32 s1km,  MetAdt *metObj_p, MetArrayAdt *metArrayObj_p);
void tcfModFunc_forwardStep(BPLUT *bp_p, MetAdt metObj, EcoStateAdt *ecoStateObj_p);
void tcfModFunc_gpp(BPLUT *bp_p, MetAdt metObj, EcoStateAdt *ecoStateObj_p);
void tcfModFunc_anaStst(BPLUT *bp_p, EcoStateAdt *ecoStateObj_p);
void tcfModFunc_kmult(BPLUT *bp_p, MetAdt metObj, EcoStateAdt *ecoStateObj_p);


// --module: l4c_PostProc9km_func.c  -----------------------------------------
        
// <JMG> 2016-03-03T17:34 new Obs min,max,mean writer and diagnostic monitor        
int l4c_PostProc9km_WriteObsMinMaxMean(hid_t dset_id,char *diagLabel_p,
           const int nCount,const float32 missingVal,float32 *avg9km_p);        
// <JMG> 2016-03-04T11:52, added to condition uint8 qa_count arrays for use by
// l4c_PostProc9km_WriteObsMinMaxMean(
int l4c_PostProc_PrepQaCountObsStats(hid_t dset_id,char *dsetName_p,
                  uint8 *uint8_9kmAry_p,float32 *wrk9kmFltCount_p);
        
// <JMG> 2014-08-21, added for debugging SEGFAULT in l4c_PostProc_fltVar9km
// on l4c_SetBits for nee_mean...
PostInvalid *l4c_GetInvalidEventCounts(void);

// <JMG> 2014-10-16T16:01, added to track PftDom and QaScore assignment
// to QA bitfield..
PostProcFreqVerifyMetrics *l4c_GetPostProcFreqVerify_PftDom(void);
PostProcFreqVerifyMetrics *l4c_GetPostProcFreqVerify_QaScore(void);

time_t l4c_PostProc9km_driverConvert(EcoStateArrayAdt *ecoStateArrayObj_p);

int l4c_PostProcessGranuleCompleteness(VarFloat32Adt *STATE_p);

// <LAJ> 7/22/14 : Moved these two protos out of the PostProc.h file.  That header file should be deleted.
// int tcfPostProc_isInCell(uint8 pft_index, float32 stateval);  // This function might not be necessary
uint8 tcf_qa_reclas(float32 error, float32 error_mn, float32 error_mx, uint32 qa_rank_mn, uint32 qa_rank_mx);

// <LAJ> 7/12/14 : Below two functions will replace 'l4c_convert_1km_to_9km_by_pft()' 
// to allow 9km summaries to be produced in the Forward Step, rather than Post Processor, routine
// void l4c_PostProc_qaRank9km(uint32 s9km, PftAdt *PFT_ARRAY, const char *datasetName_p, VarFloat32Adt *varFlt32_Obj_p);
void l4c_PostProc_qaRank9km(uint32 s9km, PftAdt *PFT_ARRAY, VarFloat32Adt *varFlt32_Obj_p);
void l4c_PostProc_fltVar9km(uint32 s9km, PftAdt *PFT_ARRAY, const char *datasetName_p, VarFloat32Adt *varFlt32_Obj_p);

// <JMG> 2014-09-04T12:47 new re-factored post-processor variable-specific handlers
int l4c_PostProcess_PrimaryVars(const char *varName_p,VarFloat32Adt *STATE_p,void *DEST_p);
int l4c_PostProcess_EC_Vars(const char *varName_p,VarFloat32Adt *STATE_p,void *DEST_p);
int l4c_PostProcess_QA(VarFloat32Adt *STATE_p,void *DEST_p);

// <LAJ> 7/22/14 : This replaces the previous '..InflateAndWrite..' functions
// deprecated file id and srcTheme since these are now fixed "L4C_OUTPUTS" etc.
int l4c_PostProc_InflateAndWrite9km(hid_t out_datatype, const int startNameIdx,
                    const char *grpName, VarFloat32Adt *STATE_p, void *DEST_p, size_t nElem);

// <LAJ> 7/22/14 : These two functions replace previous 'PostProc_Mem' functions
VarFloat32Adt l4c_PostProc9km_stepVarAlloc(const char *stateMode);
void l4c_PostProc9km_stepVarFree(const char *stateMode, VarFloat32Adt *stepVar_p);

// Destination arrays after INFLATE are stored in post9km_2DGRID_Obj_p, which is
//  same structure type as source, BUT where ALL arrays are allocated as
//  full 1624x3856 pixel rasters
int l4c_PostProc_InflateRasters(const char *dataLbl_p, 
    VarFloat32Adt *varFlt32_Obj_p,
        VarFloat32Adt *post9km_2DGRID_Obj_p) ;

void set_fillval_UUTA(void *vDest_p, const int32 dataType, const size_t atSlot);

uint8 l4c_tcf_qa_reclas(float32 error, float32 error_mn,
        float32 error_mx, uint32 qa_rank_mn, uint32 qa_rank_mx);

int l4c_hdf5_WriteOutputDataset(void *varFlt32_Ary_p,
    const int hdf5Datatype, const size_t nPixToWrite);

// <JMG> 2016-01-04T14:02 stub in during RMSE development, may re-factor yet
int l4c_PostProcess_RMSE_Alloc_Memory(void);
int l4c_PostProcess_RMSE_Release_Memory(void);

#endif
