// --------------------------------------------------------------
// module       : tcf_const.h for tcf_spin (v3.2+ and higher) constants, using new ADT names
// purpose      : Contains data structure definitions for 'tcf_spin' v3.2+ and higher code
// project      : SMAP L4C science development.
// dependencies : HDF4, FRM_API
// revision     : 2014-02-15T17:37:00-07:00, PASS 3 integration with mkgrid, mesh some constants...
// history      : 2014-02-15T10:40:00-07:00
// author       : joe glassy , Lupine Logic Inc.

// notes        : Note new for v3.2 the NESTED_1KM_IN_9KM, NESTED_3KM_IN_9KM_SQ, etc new macro names

/*
 module       : tcf_const.h
 purpose      : Contains constants and macro definitions for 'tcf_spin' program. 
 project      : SMAP L4C science development.
 dependencies :
 revision     : v3.1, Initial reformatting -- laj 09/10/13
 author       : Lucas A. Jones; lucas@ntsg.umt.edu
 notes        :

*/


#ifndef   TCF_CONST_DEF 
#define   TCF_CONST_DEF 1

#define   VERS_TAG  "v3.1-2013-09-05T18:33:00-07:00,laj"

#define   L4C_CONSOLE_ECHO       1           // Echo program messages

#define   MXSTRLEN               500         /*Max string length*/
#define   MXLISTLEN              500         /*Max list length*/
 
#define   LDOM1KM        134787240ul       // Total number of cells in EGv2 1km land domain (LDOM9KM*NCELL2)
#define   NROW1KM              14616       // Number of grid rows in EGv2 1km 
#define   NCOL1KM              34704       // Number of grid cols in EGv2 1km 

// 56359296 pixels in 2D full (4872x11568) raster, or 14976360 in 1D sparse vector form.
#define   LDOM3KM         14976360ul       // Total number of cells in EGv2 3km land domain
#define   NROW3KM               4872       // Number of grid rows in EGv2 3km 
#define   NCOL3KM              11568       // Number of grid cols in EGv2 3km 
        
#define   LDOM9KM             1664040ul    //  Total number of cells in EGv2 9km (sparse 1D) land domain

// from mkgrid's spland.h, moved this here so as not to re-define...
#define   LLAND9KM            LDOM9KM      //  Total grid cells in sparse 9km land domain  

// Full 2D grid 9km dimensions for rows,columns
#define   NROW9KM                1624       // Number of grid rows in EGv2 9km 
#define   NCOL9KM                3856       // Number of grid cols in EGv2 9km 

#define   NESTED_3KM_IN_9KM        3       // Number of 3-km cells nested along on ONE SIDE of a 9-km grid cell
#define   NESTED_1KM_IN_3KM        3       // Number of 1-km cells nested along on ONE SIDE of a 3-km grid cell
#define   NESTED_1KM_IN_9KM        9       // Number of 1-km cells nested along on ONE SIDE of a 9-km grid cell

// Define the full set of 1km, 3km, and 9km nested macro references
#define   NESTED_3KM_IN_9KM_SQ     9       // Number of TOTAL 3-km cells nested within in each 9-km grid cell
#define   NESTED_1KM_IN_3KM_SQ     9       // Number of TOTAL 1-km cells nested within in each 3-km grid cell
#define   NESTED_1KM_IN_9KM_SQ    81       // Number of TOTAL 1-km cells nested within in each 9-km grid cell
// these, just for documentation, are from mkgrid's spland.h
#ifdef USE_MKGRID_CONST
#define   NESTED_3KM_IN_9KM        3ul       // number of 3km grid cells along one side of in nested 9km domain 
#define   NESTED_3KM_IN_9KM_SQ     9ul       // Total number of 3km grid cells in nested 9km domain 
#define   NESTED_1KM_IN_9KM        9ul       // number of 1km grid cells along one side of in nested 9km domain 
#define   NESTED_1KM_IN_9KM_SQ    81ul       // Total number of 1km grid cells in nested 9km domain 
#endif
 
#define   NDAYS                  365       /* Number of days in a year*/
#define   N8DAYS                  46       /* Number of 8-day periods in a year*/
#define   NPFT                     8       /* Number of vegetated landcover types*/ 
#define   NLC                     10       /* Number of all landcover types*/
#define   NPOOL                    3       /* Number of soil organic C pools*/

#define   NMET                     8       /* Total number of input-met partial derivatives (for QA jacobian; eq. NMET_GPP + NMET_RH)*/
#define   NMET_GPP                 6       /* Number of GPP-related input-met partial derivatives (for QA jacobian)*/
#define   NMET_RH                  2       /* Number of RH-related input-met partial derivatives (for QA jacobian)*/

#define   RAD2PAR                 0.0389   /*=0.45/11.5741  convert from RAD [W m-2] to PAR [MJ m-2] */


// <JMG> note: mkgrid defines this: FILLVAL  -9999 for what its worth

#define   FILVAL_FLT             -9999.0   /*Fill value for 1km non-veg grid cells floating point data*/
#define   FILVAL_UINT8               254   /*Fill value for 1km non-veg grid cells uint8 data*/
#define   FILVAL_UINT16            65535   /*Fill value for uint16 data*/

#define   FLT_TOL                 1.0e-9   /*Floating point comparison tolerance (also for division-by-zero)*/

/*Starting tolerance to begin spinup interations [g C m-2 d-1].*/
#define   STARTING_SPINUP_TOL     1.0e4    /*Some arbitrarily high floating point number, must be greater than stopping tolerance*/   

//  directives for Joe's L4C_BPLUT function
#define MAX_LINE_BUFFER_LEN       512
#define STRING_MATCHED            0

#define BPLUT_N_EXPECTED_RECORDS  8
#define BPLUT_N_EXPECTED_FIELDS   44    // LAJADD

// MACROS DEFINED IN tcf_const.h for tcf_spin_v3.2 look like this:
//     2D offsets: y is outermost, x is innermost (fastest varying) term
#define M_2D_B0(x,y,n_y) (((x)*n_y)+(y))
//     Serialization offset for nested 1km 2D grid within 9km 2D grid
//     indices x,y correspond to 9km grid, while i,j is the nested cell reference
#define M_2D_N9_B0(x,y,i,j,ny) (((x)*9 + (i))*ny + (y)*9 + (j))

/*Converts 0-45 day index of 8-day periods into day-of-year*/
#define DAY8_to_DOY(i)  ((i+1)*8 - 7)

// floating point round() macro from "The Standard C", P.J.Plaugher 
// TBD: deprecate, uses FRM_API's now
#define R_ROUND(x) (((x)<0.50)?ceil(x-0.5):floor(x+0.50))


#endif 

