// --------------------------------------------------------------
// module       : tcf_func.h LEAD_FORK with new ADT names, at >= PASS 3 maturity now
// purpose      : Contains function prototypes lead-fork of 'tcf_spin' v3.2 program.
// project      : SMAP L4C science development.
// dependencies : HDF4, FRM_API
// revision     : 2014-02-17t11:46, migrated array_io.h prototypes into tcf_func.h
// history      : 2014-02-15T10:40:00-07:00
// author       : joe glassy , Lupine Logic Inc.
// notes        : The "lead_fork" configuration of tcf_spin_v3.2 implements new
//                ADT namespaces, consolidates all C and .h headers in one directory
//                and serves as the middle (transitional) codebase betweeen the 
//                ORIGINAL code and the final L4_DAILY production codebase.


/*
 module       : tcf_func.h
 purpose      : Contains function prototypes for 'tcf_spin' program.
 project      : SMAP L4C science development.
 dependencies : 
 revision     : v3.1, Initial reformatting -- laj 09/10/13
 author       : Lucas A. Jones; lucas@ntsg.umt.edu
 notes        :

*/

#ifndef TCF_FUNC_DEF 
#define TCF_FUNC_DEF 1

// In file 'tcfMem.c' --  migrating to l4c_DAILY_prot.h
// EcoStateArrayAdt tcfMem_stateArrayAlloc(uint32 nelem1km);

int tcfMem_loadDailyMet(uint32 doy, uint32 *t8day, MetArrayAdt *metArrayObj_p);
void tcfMem_loadStaticMet(MetArrayAdt *metArrayObj_p);

void tcfMem_loadCstate(char pname[], char *doyStr, EcoStateArrayAdt *ecoStateArrayObj_p);
void tcfMem_loadFtRollingState(char *doyStr, EcoStateArrayAdt *ecoStateArrayObj_p);

PftAdt tcfMem_pftArrayAllocInit(uint32 nelem1km, uint32 offset);

void tcfMem_saveCstate(char pname[], char *doyStr, uint32 nelem, EcoStateArrayAdt *ecoStateArrayObj_p);
void tcfMem_saveFtRollingState(char *doyStr, uint32 nelem, EcoStateArrayAdt *ecoStateArrayObj_p);

// In file 'tcfIo.c':
int tcfIo_params(char *fname);

// In file 'tcfModMult.c':
float64 *tcfModMult_ramp(float64 x, float32 xmin, float32 xmax);
float64 *tcfModMult_rhTsoil(BPLUT *bp_p, float64 tsoil);
float64 tcfModMult_rescalSmrz(float32 smrz, float32 smrz_mn, float32 smrz_mx);

// In file 'tcfModFunc.c':  -- now in l4c_DAILY_prot.h


// In file 'tcfModQa.c':
void tcfModQa_jac(BPLUT *bp_p, MetAdt metObj, EcoStateAdt *ecoStateObj_p);
void tcfModQa_step(BPLUT *bp_p, MetAdt metObj, EcoStateAdt *ecoStateObj_p);
float64 tcfModQa_rescal(uint8 qa_rank, uint32 qa_rank_mn, uint32 rank_mx, float64 error_mn, float64 error_mx);
uint8 tcfModQa_reclas(float64 error, float64 error_mn, float64 error_mx, uint32 qa_rank_mn, uint32 qa_rank_mx);

// In file 'tcfModDriverSubset.c':  -- not yet enabled
//void tcfModDriverSubset_numSpin(PftAdt PFT, MetArrayAdt metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);
//void tcfModDriverSubset_anaPrecond(PftAdt PFT, MetArrayAdt metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);
//void tcfModDriverSubset_forwardRun(PftAdt PFT, MetArrayAdt metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);

// In file 'tcfModDriverGrid.c':
void tcfModDriverGrid_anaPrecond(PftAdt pftObj, MetArrayAdt metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);

void tcfModDriverGrid_anaPrecond_stst(PftAdt pftObj, MetArrayAdt metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);

void tcfModDriverGrid_numSpin(PftAdt pftObj, MetArrayAdt metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);

uint32 tcfModDriverGrid_numSpin_tols(float32 *max_tol, PftAdt pftObj, MetArrayAdt metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);

void tcfModDriverGrid_forwardSpin(uint32 doyStop, uint32 spin_iter, PftAdt pftObj, MetArrayAdt metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);

void tcfModDriverGrid_forwardRun(PftAdt pftObj, MetArrayAdt metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);

// void tcfModDriverGrid_forwardStep(uint32 doy, PftAdt pftObj, MetArrayAdt metArrayObj_p, EcoStateArrayAdt *ecoStateArrayObj_p);

//  In file 'tcfAction.c' 
void tcfAction_anaSpin(void);
void tcfAction_numSpin(void);
void tcfAction_seasAdj(void);
void tcfAction_forwardRun(void); 
void tcfAction_forwardStep(void);

// In file 'tcfSession.c'
void tcfSession_init(void);
void tcfSession_close(void);
void tcfSession_abort(void);

// In file 'l4c_LocalMsg.c'
void l4c_LocalMsg(FILE *logf,char *fmtMsg_p,...);

//  In file 'L4C_bplut.c' given to me by Joe
BPLUT *L4C_Read_BPLUT(const char *bpLutPath, int *nBplutRecords);
BPLUT *L4C_GetBPLUT_Private(void);
BPLUT *L4C_GetBPLUT_Record(const char *landcoverLutKey_p, const int nValidBPLUT_Records);
BPLUT *L4C_GetBPLUT_ByIndex(const int landcoverIntegerKey, const int nValidBPLUT_Records);

// In file "array_io.c"  (deprecated array_io.h to put its prototypes in here)
char *fpath_cat(int nargs, ...);

int load_part_uint8(char fname[], uint8 x[], uint32 offset, uint32 n);
int load_part_uint16(char fname[], uint16 x[], uint32 offset, uint32 n);
int load_part_flt32(char fname[], float32 x[], uint32 offset, uint32 n);

int load_int8(char fname[], int8 x[], uint32 n);
int load_uint8(char fname[], uint8 x[], uint32 n);
int load_int16(char fname[], int16 x[], uint32 n);
int load_uint16(char fname[], uint16 x[], uint32 n);
int load_uint32(char fname[], uint32 x[], uint32 n);
int load_flt32(char fname[], float32 x[], uint32 n);
int load_flt64(char fname[], float64 x[], uint32 n);

int write_int8(char fname[], int8 x[], uint32 n);
int write_uint8(char fname[], uint8 x[], uint32 n);
int write_int16(char fname[], int16 x[], uint32 n);
int write_uint16(char fname[], uint16 x[], uint32 n);
int write_uint32(char fname[], uint32 x[], uint32 n);
int write_flt32(char fname[], float32 x[], uint32 n);
int write_flt64(char fname[], float64 x[], uint32 n);



#endif
